SystemOrganization addCategory: #'Fuel-Tests'!
SystemOrganization addCategory: #'Fuel-Core'!
SystemOrganization addCategory: #'Fuel-Core-Cluster'!
SystemOrganization addCategory: #'Fuel-Core-Analyzer'!
SystemOrganization addCategory: #'Fuel-Core-Errors'!

Trait named: #FLTClassParameter
	uses: {}
	category: 'Fuel-Core-Cluster'!

!FLTClassParameter classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/19/2011 12:36'!
for: aClass

	^ self basicNew
		parameterClass: aClass;
		yourself.! !

!FLTClassParameter methodsFor: 'comparing' stamp: 'MartinDias 4/19/2011 12:36'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ self parameterClass = anObject parameterClass! !

!FLTClassParameter methodsFor: 'comparing' stamp: 'MartinDias 4/19/2011 12:36'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ self parameterClass hash! !

!FLTClassParameter methodsFor: 'accessing' stamp: 'MartinDias 4/19/2011 12:35'!
parameterClass

	self explicitRequirement ! !

!FLTClassParameter methodsFor: 'accessing' stamp: 'MartinDias 4/19/2011 12:35'!
parameterClass: aClass

	self explicitRequirement ! !

!FLTClassParameter methodsFor: 'printing' stamp: 'MartinDias 4/19/2011 12:36'!
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printOn: aStream.
	aStream nextPut: $(.
	self parameterClass printOn: aStream.
	aStream nextPut: $).
	! !

Trait named: #FLTCreateClassOrTraitSerializationTest
	uses: {}
	category: 'Fuel-Tests'!

!FLTCreateClassOrTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire class or trait, in the cases in which they *are not* present in the image at materialization time.!

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 15:55'!
resultOfSerializeRemoveAndMaterialize: aClass

	| stream |
	stream := self internalSerializationOf: aClass.
	aClass removeFromSystem.
	^self materializationOn: stream reset.
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 15:55'!
resultOfSerializeRemoveAndMaterializeAll: classes

	| stream |
	stream := self internalSerializationOfAll: classes.
	classes do: [ :aClass | aClass removeFromSystem ].
	^self materializationOn: stream reset.
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/26/2011 01:38'!
testCreateBasic
	"Tests materialization a class or trait not defined in the image."

	| aClassOrTrait materializedClassOrTrait environment category name |
	aClassOrTrait := self newClassOrTrait.
	environment := aClassOrTrait environment.
	category := aClassOrTrait category.
	name := aClassOrTrait name.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: environment == materializedClassOrTrait environment.
	self assert: category equals: materializedClassOrTrait category.
	self assert: name equals: materializedClassOrTrait name.

    "It is important to notice that (at this moment) Fuel adds the materialized class or trait into Smalltalk globals"
    self assert: (Smalltalk at: name) equals: materializedClassOrTrait.
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/28/2011 02:57'!
testCreateWithClassSideMethod
	"Tests materialization of a class-side method in a class or trait not defined in the image."

	| aClassOrTrait materializedClassOrTrait |
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait classSide compile: 'fortyTwo ^42' classified: #test.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: (materializedClassOrTrait classSide includesSelector: #fortyTwo).
	self assert: #(fortyTwo) asSet equals: materializedClassOrTrait classSide localSelectors.
	self assert: #test equals: (materializedClassOrTrait classSide whichCategoryIncludesSelector: #fortyTwo).
	self assert: 42 equals: ((self newInstanceFrom: materializedClassOrTrait) class perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/27/2011 23:42'!
testCreateWithClassTrait
	"Tests materialization of a class not defined in the image, with a class trait"
	
	| aClassOrTrait materializedClassOrTrait aTrait |	
	aTrait := self stubTrait.
	aTrait classSide compile: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait addToComposition: aTrait.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 1 equals: (materializedClassOrTrait traits size).
	self assert: (materializedClassOrTrait traits includes: aTrait).
	self assert: (aTrait users includes: materializedClassOrTrait).
	self assert: materializedClassOrTrait classSide localSelectors isEmpty.
	self assert: 42 equals: ((self newInstanceFrom: materializedClassOrTrait) class perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/25/2011 20:35'!
testCreateWithComment
	"Tests materialization of the comment of a class or trait not defined in the image."
	
	| aClassOrTrait materializedClassOrTrait |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait comment: 'test comment' stamp: 'test stamp'.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 'test comment' equals: materializedClassOrTrait comment asString.
	self assert: 'test stamp' equals: materializedClassOrTrait organization commentStamp.! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 4/11/2011 18:13'!
testCreateWithFullRemoteString
	"Tests materialization using full RemoteString serialization."
	
	| aClassOrTrait materializedClassOrTrait commentSourcePointer methodSourcePointer materializedCommentSourcePointer materializedMethodSourcePointer |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait comment: 'test comment'.
	aClassOrTrait compile: 'fortyTwo ^42'.
	commentSourcePointer := aClassOrTrait organization commentRemoteStr sourcePointer.
	methodSourcePointer := (aClassOrTrait >> #fortyTwo) sourcePointer.

	FLRemoteStringClusterSerializer setFullSerialization.	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.
	FLRemoteStringClusterSerializer reset.
	materializedCommentSourcePointer := materializedClassOrTrait organization commentRemoteStr sourcePointer.
	materializedMethodSourcePointer := (materializedClassOrTrait >> #fortyTwo) sourcePointer.

	self deny: commentSourcePointer = materializedCommentSourcePointer.
	self deny: methodSourcePointer = materializedMethodSourcePointer.
	self assert: 'test comment' equals: materializedClassOrTrait comment.! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/10/2011 23:44'!
testCreateWithLightRemoteString
	"Tests materialization using light RemoteString serialization."
	
	| aClassOrTrait materializedClassOrTrait commentSourcePointer methodSourcePointer materializedCommentSourcePointer materializedMethodSourcePointer |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait comment: 'test comment'.
	aClassOrTrait compile: 'fortyTwo ^42'.
	commentSourcePointer := aClassOrTrait organization commentRemoteStr sourcePointer.
	methodSourcePointer := (aClassOrTrait >> #fortyTwo) sourcePointer.

	FLRemoteStringClusterSerializer setLightSerialization.	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.
	FLRemoteStringClusterSerializer reset.
	materializedCommentSourcePointer := materializedClassOrTrait organization commentRemoteStr sourcePointer.
	materializedMethodSourcePointer := (materializedClassOrTrait >> #fortyTwo) sourcePointer.

	self assert: commentSourcePointer = materializedCommentSourcePointer.
	self assert: methodSourcePointer = materializedMethodSourcePointer.	
	self assert:  'test comment' equals: materializedClassOrTrait comment.! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/28/2011 02:57'!
testCreateWithMethod
	"Tests materialization of a compiled method in a class not defined in the image."
	
	| aClassOrTrait materializedClassOrTrait |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait compile: 'fortyTwo ^42' classified: #test.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: (materializedClassOrTrait includesSelector: #fortyTwo).
	self assert: #(fortyTwo) asSet equals: materializedClassOrTrait localSelectors.
	self assert: #test equals: (materializedClassOrTrait whichCategoryIncludesSelector: #fortyTwo).
	self assert: 42 equals: ((self newInstanceFrom: materializedClassOrTrait) perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/27/2011 23:39'!
testCreateWithTrait
	"Tests materialization of a class not defined in the image, with a trait"
	
	| aClassOrTrait materializedClassOrTrait aTrait |	
	aTrait := self stubTrait.
	aTrait compile: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait addToComposition: aTrait.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 1 equals: (materializedClassOrTrait traits size).
	self assert: (materializedClassOrTrait traits includes: aTrait).
	self assert: (aTrait users includes: materializedClassOrTrait).
	self assert: materializedClassOrTrait localSelectors isEmpty.
	self assert: 42 equals: ((self newInstanceFrom: materializedClassOrTrait) perform: #fortyTwo).
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/27/2011 23:41'!
testCreateWithTraitOnClassSide
	"Tests materialization of a class not defined in the image, with a trait added to its class side"
	
	| aClassOrTrait materializedClassOrTrait aTrait |	
	aTrait := self stubTrait.
	aTrait compile: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait classSide addToComposition: aTrait.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 1 equals: (materializedClassOrTrait classSide traits size).
	self assert: (materializedClassOrTrait classSide traits includes: aTrait).
	self assert: (aTrait users includes: materializedClassOrTrait classSide).
	self assert: materializedClassOrTrait classSide localSelectors isEmpty.
	self assert: 42 equals: ((self newInstanceFrom: materializedClassOrTrait) class perform: #fortyTwo).! !

Trait named: #FLTExternalClassOrTraitSerializationTest
	uses: {}
	category: 'Fuel-Tests'!

!FLTExternalClassOrTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of classes and traits as *external* objects, i.e. the classes or traits have to be present in the image at materialization time.!

!FLTExternalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 16:47'!
testExternal
	"Tests that serialization of the class or trait preserves identity"
	
	self assertSerializationIdentityOf: self newClassOrTrait! !

!FLTExternalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 16:46'!
testExternalClassSide
	"Tests that serialization of the class side preserves identity"
	
	self assertSerializationIdentityOf: self newClassOrTrait classSide ! !

!FLTExternalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 16:49'!
testMaterializeAfterRemove
	"Tests that serializer does not tolarate when the class was removed between serialization and materialization"
	
	| aClass stream |
	aClass := self newClassOrTrait.
	
	stream := self serializationOf: aClass.
	aClass removeFromSystem.
	
	self should: [self materializationOn: stream reset] raise: ClassNotFound.! !

!FLTExternalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 16:49'!
testMaterializeAfterRename
	"Tests that serializer does not tolarate when the class was renamed between serialization and materialization"
	
	| aClass stream |
	aClass := self newClassOrTrait.
	
	stream := self serializationOf: aClass.
	aClass rename: (aClass name, 'Renamed') asSymbol.
	
	self should: [self materializationOn: stream reset] raise: ClassNotFound.! !

Trait named: #FLTInternalSerializationTest
	uses: {}
	category: 'Fuel-Tests'!

!FLTInternalSerializationTest commentStamp: '<historical>' prior: 0!
I implement shared behavior for serialization tests when serializing the entire class or trait.!

!FLTInternalSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/23/2011 18:23'!
internalSerializationOf: aClass
	
	| analyzer |
	analyzer := FLAnalyzer newDefaultInternalObjects: {aClass}.
	
	^ (self serializer
		serialize: aClass analyzer: analyzer;
		stream)! !

!FLTInternalSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/23/2011 18:23'!
internalSerializationOfAll: classes
		
	^ (self serializer
		serialize: classes 
		analyzer: (FLAnalyzer newDefaultInternalObjects: classes);
		stream)! !

Trait named: #FLTSerializationTest
	uses: {}
	category: 'Fuel-Tests'!

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 16:21'!
setUp

	super setUp.
	self setUpStream.
	self setUpClassFactory.! !

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 16:20'!
setUpClassFactory

	self classFactory: ClassFactoryForTestCase new! !

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 03:19'!
setUpStream

	self streams: OrderedCollection new.! !

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 16:21'!
tearDown

	super tearDown.
	self tearDownStream.
	self tearDownClassFactory.! !

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 16:20'!
tearDownClassFactory

	self classFactory cleanUp! !

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 03:20'!
tearDownStream

	self streams do: [ :aFile | aFile close ].
! !

!FLTSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 03:24'!
writeStream 

"	^ (RWBinaryOrTextStream on: '') binary"
	^ self streams add: (StandardFileStream forceNewFileNamed: 'testTemporal.fuel') binary! !

!WordArrayForSegment methodsFor: '*Fuel' stamp: 'MartinDias 4/19/2011 14:04'!
fuelSerializer

	^ FLRawArrayedCollectionClusterSerializer for: self class! !

!LargeNegativeInteger methodsFor: '*Fuel' stamp: 'TristanBourgois 9/24/2010 15:09'!
fuelSerializer 

	^ FLLargeNegativeIntegerClusterSerializer instance.! !

!ByteSymbol methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:12'!
fuelAccept: aVisitor

	aVisitor visitPrimitive: self! !

!ByteSymbol methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:13'!
fuelSerializer

	^ FLSymbolClusterSerializer instance! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:33'!
fuelAccept: aVisitor

	aVisitor visitObject: self.! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 10/22/2010 12:56'!
fuelSerializer 

	^ (self isNamedPrimitive)
		ifTrue:  [ FLNamedPrimitiveMethodClusterSerializer instance ]
		ifFalse: [ FLCompiledMethodClusterSerializer instance ]
! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 10/22/2010 12:56'!
isNamedPrimitive 

	^ self primitive = 117! !

!Character methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:28'!
fuelAccept: aVisitor

	aVisitor visitPrimitive: self! !

!Character methodsFor: '*Fuel' stamp: 'MartinDias 9/23/2010 17:41'!
fuelSerializer

	^ FLCharacterClusterSerializer instance! !

!ByteArray methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:06'!
fuelAccept: aVisitor

	aVisitor visitPrimitive: self! !

!ByteArray methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 14:38'!
fuelSerializer 

	^ FLByteArrayClusterSerializer instance! !

Error subclass: #ClassNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 10/12/2010 10:54'!
hasInstanceVariables
	"Returns whether the receiver has instance variables"
	
	^ self instSize > 0 ! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 3/28/2011 02:39'!
initializeTraitComposition: aTraitComposition

	aTraitComposition assertValidUser: self.
	self traitComposition: aTraitComposition.
	aTraitComposition traits do: [:each | each addUser: self]! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 3/17/2011 16:36'!
printCompactHierarchy
	"Returns a short description of the class hierarchy"
	
	^ self withAllSuperclasses inject: '' into: [:a :b | b printString, '.', a ]! !

!UndefinedObject methodsFor: '*Fuel' stamp: 'MartinDias 10/19/2010 10:50'!
fuelAccept: aVisitor

	aVisitor visitNilTrueFalse: self.! !

!UndefinedObject methodsFor: '*Fuel' stamp: 'TristanBourgois 9/24/2010 13:56'!
fuelSerializer 

	^ FLNilTrueFalseClusterSerializer instance! !

!Integer methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:26'!
fuelAccept: aVisitor

	aVisitor visitPrimitive: self! !

!OrderedCollection methodsFor: '*Fuel' stamp: 'MartinDias 10/12/2010 16:11'!
hasAnyDuplicate
	"Answers if there is an element added more than one time"

	^ self size ~= self asSet size ! !

!Boolean methodsFor: '*Fuel' stamp: 'MartinDias 10/19/2010 10:50'!
fuelAccept: aVisitor

	aVisitor visitNilTrueFalse: self.! !

!Boolean methodsFor: '*Fuel' stamp: 'MartinDias 9/27/2010 09:15'!
fuelSerializer 

	^ FLNilTrueFalseClusterSerializer instance ! !

!RemoteString methodsFor: '*Fuel' stamp: 'MartinDias 3/21/2011 17:41'!
fuelSerializer 

	^FLRemoteStringClusterSerializer configuredInstance.! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:23'!
fuelAccept: aVisitor

	aVisitor visitClassOrTrait: self! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 3/24/2011 03:34'!
fuelSerializer 

	^FLTraitClusterSerializer instance! !

!Float methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:27'!
fuelAccept: aVisitor

	aVisitor visitPrimitive: self! !

!Float methodsFor: '*Fuel' stamp: 'MartinDias 10/20/2010 22:46'!
fuelSerializer 

	^ FLFloatClusterSerializer instance! !

!ColorArray methodsFor: '*Fuel' stamp: 'MartinDias 4/19/2011 14:04'!
fuelSerializer

	^ FLRawArrayedCollectionClusterSerializer for: self class! !

!Metaclass methodsFor: '*Fuel' stamp: 'MartinDias 10/19/2010 10:50'!
fuelAccept: aVisitor

	aVisitor visitMetaclass: self! !

!Metaclass methodsFor: '*Fuel' stamp: 'MartinDias 3/23/2011 18:10'!
fuelSerializer 
	^ FLClassSideClusterSerializer instance.! !

!ClassTrait methodsFor: '*Fuel' stamp: 'MartinDias 3/23/2011 18:10'!
fuelSerializer 
	^ FLClassSideClusterSerializer instance.! !

!Bitmap methodsFor: '*Fuel' stamp: 'MartinDias 4/19/2011 14:03'!
fuelSerializer

	^ FLRawArrayedCollectionClusterSerializer for: self class! !

!SmallInteger methodsFor: '*Fuel' stamp: 'MartinDias 12/28/2010 14:12'!
fuelSerializer

	^ FLSmallIntegerClusterSerializer for: self! !

!TraitBehavior methodsFor: '*Fuel' stamp: 'MartinDias 3/28/2011 02:45'!
initializeTraitComposition: aTraitComposition

	self traitComposition: aTraitComposition.
	aTraitComposition traits do: [:each | each addUser: self]! !

TestCase subclass: #FLClusterSerializerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLClusterSerializerTest methodsFor: 'tests' stamp: 'MartinDias 10/12/2010 16:10'!
testUniqueIds

	" 
	Inspect the mapping:
	FLClusterSerializer allSubclasses collect: [ :aClass | aClass->([aClass id] ifError: [nil]) ]
	"

	self shouldnt: [ FLClusterSerializer validateIdDistribution ] raise: Error! !

TestCase subclass: #FLSerializationTest
	uses: FLTSerializationTest
	instanceVariableNames: 'streams classFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLSerializationTest commentStamp: '<historical>' prior: 0!
I have black-box tests for testing that objects serializes and materializes well.!

FLSerializationTest class
	uses: FLTSerializationTest classTrait
	instanceVariableNames: ''!

FLSerializationTest subclass: #FLBasicSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/25/2010 14:24'!
testArray
	self assertSerializationEqualityOf: #(10 20)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/25/2010 14:27'!
testByteArrays

	self assertSerializationEqualityOf: { #[1 2 3 4 5 6]. #[1 2 3 4 5 6]. #[7 8 9 10 11 12]. #[7 8 9 10 11 12] }! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:00'!
testCharacter

	self assertSerializationIdentityOf: $a! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 4/11/2011 13:09'!
testClassVariable

	| aClass | 
	aClass := self classFactory newClass.
	aClass addClassVarNamed: 'ClassVariable'.
	self assertSerializationIdentityOf: (aClass classPool associationAt: #ClassVariable).! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 4/19/2011 13:02'!
testColorArray

	self assertSerializationEqualityOf: (ColorArray with: (Color r: 0.0 g: 0.031 b: 0.008) with: (Color r: 0.0 g: 0.07 b: 0.023))! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/21/2011 16:08'!
testCyclicLink

	| a materializedA |
	a := Link new.
	a nextLink: a.
	
	materializedA := self resultOfSerializeAndMaterialize: a.

	"We can't check using the equality of the links because hangs ad infinitum. So we only check that the structure is right."
	self assert: materializedA nextLink == materializedA! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/11/2010 19:31'!
testCyclicLinks

	| a b c materializedA |
	a := Link new.
	b := Link new.
	c := Link new.
	
	a nextLink: b.
	b nextLink: c.
	c nextLink: a.
	
	materializedA := self resultOfSerializeAndMaterialize: a.

	"We can't check using the equality of the links because hangs ad infinitum. So we only check that the structure is right."
	self assert: materializedA nextLink nextLink nextLink == materializedA.
	self deny: materializedA nextLink == materializedA.
	self deny: materializedA nextLink nextLink == materializedA.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:59'!
testDictionary

	self assertSerializationEqualityOf: (Dictionary with: 1->2).! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/26/2010 19:46'!
testEmptyArray
	self assertSerializationEqualityOf: #()! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/26/2010 19:45'!
testEmptyDictionary

	self assertSerializationEqualityOf: Dictionary new! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/26/2010 19:46'!
testEmptyOrderedCollection

	self assertSerializationEqualityOf: OrderedCollection new! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:59'!
testFalse

	self assertSerializationIdentityOf: false! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/21/2010 10:49'!
testFloat
	
	self assertSerializationEqualityOf: 180.0! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/2/2010 13:01'!
testGradientFillStyle
	self assertSerializationEqualityOf: GradientFillStyle sample! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:54'!
testLargeNegativeInteger

	self assertSerializationEqualityOf: -354314316134313999999999.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:54'!
testLargePositiveInteger

	self assertSerializationEqualityOf: 354314316134313999999999.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:56'!
testNil

	self assertSerializationIdentityOf: nil! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/29/2010 11:27'!
testOrderedCollection

	self assertSerializationEqualityOf: (OrderedCollection with: 10 with: 20)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/25/2010 18:40'!
testPair

	self assertSerializationEqualityOf:
			(FLPair new
				left: 10;
				right: 20;
				yourself)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:57'!
testPoint

	self assertSerializationEqualityOf: 10@20! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/26/2010 14:57'!
testRectangle
	
	self assertSerializationEqualityOf: (10@20 corner: 30@40)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/21/2011 17:29'!
testRemoteStringFullSerialization
	"Tests that the materialized RemoteString has a new location on file."

	| aRemoteString materializedRemoteString |	
	aRemoteString := RemoteString new setSourcePointer: thisContext method sourcePointer.

	FLRemoteStringClusterSerializer setFullSerialization.
	materializedRemoteString := self resultOfSerializeAndMaterialize: aRemoteString.
	FLRemoteStringClusterSerializer reset.

	self assert: aRemoteString string equals: materializedRemoteString string.
	self deny: aRemoteString sourcePointer == materializedRemoteString sourcePointer.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/21/2011 17:28'!
testRemoteStringLightSerialization
	"Tests that the materialized RemoteString has *the same* location on file."

	| aRemoteString materializedRemoteString |	
	aRemoteString := RemoteString new setSourcePointer: thisContext method sourcePointer.

	FLRemoteStringClusterSerializer setLightSerialization.
	materializedRemoteString := self resultOfSerializeAndMaterialize: aRemoteString.
	FLRemoteStringClusterSerializer reset.

	self assert: aRemoteString string equals: materializedRemoteString string.
	self assert: aRemoteString sourcePointer == materializedRemoteString sourcePointer.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:57'!
testSmallInteger

	self assertSerializationIdentityOf: SmallInteger one.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:57'!
testSmallIntegerMaxValue

	self assertSerializationIdentityOf: SmallInteger maxVal .
	! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:04'!
testSmallIntegerMinValue

	self assertSerializationIdentityOf: SmallInteger minVal ! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/29/2010 11:27'!
testSmallIntegerNegative

	self assertSerializationIdentityOf: -42.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/17/2011 20:54'!
testSmalltalkGlobal

	self assertSerializationIdentityOf: Smalltalk! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/18/2010 12:23'!
testSmalltalkGlobalAssociation

	self assertSerializationIdentityOf: (Smalltalk associationAt: #Transcript)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:58'!
testString

	self assertSerializationEqualityOf: 'testString'! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:58'!
testSymbol

	self assertSerializationIdentityOf: #testSymbol! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:59'!
testTrue

	self assertSerializationIdentityOf: true! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/21/2011 16:10'!
testWordArray

	self assertSerializationEqualityOf: (WordArray with: 10 with: 20)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/21/2011 16:10'!
testWordArrayForSegment 

	self assertSerializationEqualityOf: (WordArrayForSegment with: 1929386344 with: 214577675)! !

FLSerializationTest subclass: #FLClassSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLClassSerializationTest commentStamp: '<historical>' prior: 0!
I have the common behavior for testing class serialization.!

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 23:37'!
compiledMethodStubClass
	"Returns a class for testing"

	^ CompiledMethod subclass: self stubClassName 
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category.
! !

!FLClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 17:42'!
newClassOrTrait

	^ self stubClass! !

!FLClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 17:42'!
newInstanceFrom: aClass

	^ aClass new! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/21/2011 18:33'!
stubClass
	"Returns a class for testing"

	^ self stubClass: self stubClassName superclass: self stubSuperclass
! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/21/2011 18:30'!
stubClass: name superclass: superclass
	"Returns a class for testing"

	^ superclass subclass: name instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: self class category! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 20:40'!
stubClassName
	"The name for the testing class"

	^ #FLStubClass! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/18/2011 14:46'!
stubClassWithInstanceVars: instanceVariableNames
	"Returns a class for testing, with the specified instance variables."

	^ self stubSuperclass subclass: self stubClassName 
		instanceVariableNames: instanceVariableNames
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category.
! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/18/2011 14:36'!
stubSuperclass

	^Object! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/23/2011 16:31'!
stubTrait
	"Returns a trait for testing"

	^ Trait named: self stubTraitName uses: {} category: self class category! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 20:40'!
stubTraitName

	^#FLStubTrait! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/21/2011 00:49'!
tearDown

	super tearDown.
	Smalltalk allClassesAndTraits select: [ :classOrTrait | classOrTrait name beginsWith: 'FLStub' ]
		thenDo: [ :classOrTrait | classOrTrait removeFromSystem ].

! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 23:16'!
variableByteStubClass
	"Returns a class for testing"

	^ Object variableByteSubclass: self stubClassName 
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category.
! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 23:03'!
variableStubClass
	"Returns a class for testing"

	^ self stubSuperclass variableSubclass: self stubClassName 
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category.
! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 23:11'!
variableWordStubClass
	"Returns a class for testing"

	^ Object variableWordSubclass: self stubClassName 
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category.
! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/20/2011 23:03'!
weakStubClass
	"Returns a class for testing"

	^ self stubSuperclass weakSubclass: self stubClassName 
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category.
! !

FLClassSerializationTest subclass: #FLCreateClassSerializationTest
	uses: FLTInternalSerializationTest + FLTCreateClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLCreateClassSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire class, in the cases in which the class *is not* present in the image at materialization time.!

FLCreateClassSerializationTest class
	uses: FLTCreateClassOrTraitSerializationTest classTrait + FLTInternalSerializationTest classTrait
	instanceVariableNames: ''!

!FLCreateClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/18/2011 14:38'!
stubSuperclass

	^FLPair! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/29/2011 00:35'!
testCorrectClassTypeFor: aClass
	| materializedClass |	
	
	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.
	
	self deny: aClass == materializedClass.
	self assert: aClass superclass == materializedClass superclass.
	self assert: self stubClassName equals: materializedClass name.
	self assert: aClass typeOfClass equals: materializedClass typeOfClass.
	self assert: aClass format equals: materializedClass format.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/27/2011 22:58'!
testCreateClassAndMetaclass
	"Tests materialization a class or trait not defined in the image and that the metaclass is also created."

	| aClassOrTrait materializedClassOrTrait environment category name metaclassName metaclass |
	aClassOrTrait := self newClassOrTrait.
	environment := aClassOrTrait environment.
	category := aClassOrTrait category.
	name := aClassOrTrait name.
	metaclassName := aClassOrTrait class name.
	metaclass := aClassOrTrait class.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.
	
	self assert: environment == materializedClassOrTrait environment.
	self assert: category equals: materializedClassOrTrait category.
	self assert: name equals: materializedClassOrTrait name.
	"Notice that the metaclass is serialized by Fuel and a new one will be created."
	self assert: metaclassName equals: materializedClassOrTrait class name.
	self deny: metaclass == materializedClassOrTrait class.

    "It is important to notice that (at this moment) Fuel adds the materialized class or trait into Smalltalk globals"
    self assert: (Smalltalk at: name) equals: materializedClassOrTrait.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 4/11/2011 13:08'!
testCreateClassWithClassSideVariable
	"Tests materialization a class not defined in the image, with a class-side instance variable."
	
	| aClass materializedClass |	
	aClass := self stubClass.
	aClass class addInstVarNamed: 'a'.
	aClass instVarNamed: 'a' put: #test.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 equals: materializedClass class instVarNames size.
	self assert: (materializedClass class instVarNames includes: 'a').
	self assert: #test equals: (materializedClass instVarNamed: 'a').! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 4/11/2011 13:09'!
testCreateClassWithClassVariable
	"Tests materialization a class not defined in the image, with a class variable."
	
	| aClass materializedClass |	
	aClass := self stubClass.
	aClass addClassVarNamed: 'A'.
	aClass classPool at: 'A' put: #test.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 equals: materializedClass classVarNames size.
	self assert: (materializedClass classVarNames includes: #A).
	self assert: #test equals: (materializedClass classPool at: #A).! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/24/2011 15:09'!
testCreateClassWithCreatedTrait
	"Tests materialization of a class and a trait not defined in the image. The class uses the trait."
	
	| aClass aTrait result materializedClass materializedTrait |	
	aClass := self stubClass.
	aTrait := self stubTrait.
	aClass addToComposition: aTrait.

	result := self resultOfSerializeRemoveAndMaterializeAll: {aClass. aTrait}.
	materializedClass := result first.
	materializedTrait := result second.

	self assert: 2 equals: result size.
	self assert: (materializedClass traits first == materializedTrait).
	self assert: (materializedTrait users includes: materializedClass).! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 13:51'!
testCreateClassWithInstance
	"Tests materialization of a class who references an instance of itself."
	
	| aClass anInstance materializedClass materializedInstance |
	aClass := self stubClassWithInstanceVars: 'name'.
	aClass class instanceVariableNames: 'instance'.
	
	anInstance := aClass new
		instVarNamed: 'name' put: #testName;
		yourself.
	aClass 
		instVarNamed: 'instance' 
		put: anInstance.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.
	materializedInstance := materializedClass instVarNamed: 'instance'.
	
	self deny: aClass == materializedClass.
	self deny: anInstance == materializedInstance.
	self assert: #testName equals: (materializedInstance instVarNamed: 'name').! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 02:23'!
testCreateClassWithSharedPool
	"Tests materialization of a class not defined in the image, with a shared pool (NOTE: it will be an identity problem if it isn't a Smalltalk global')"
	
	| aClass materializedClass |	
	aClass := self stubClass.
	aClass addSharedPool: TextConstants.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 equals: (materializedClass sharedPools size).
	self assert: TextConstants == materializedClass sharedPools first.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/20/2011 08:59'!
testCreateClassWithVariables
	"Tests materialization of a class not defined in the image, with instance variables."
	
	| aClass materializedClass |	
	aClass := self stubClassWithInstanceVars: 'var1 var2'.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: aClass instVarNames equals: materializedClass instVarNames.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 18:18'!
testCreateCompiledMethodClass
	"Tests materialization of a CompiledMethod class that is not defined in the image"
	self testCorrectClassTypeFor: self compiledMethodStubClass.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/21/2011 18:28'!
testCreateHierarchy
	"Tests materialization of a hierarchy of classes that is not defined in the image"
	
	| serializedClasses materializedClasses a b c d |	
	a := self stubClass: #FLStubA superclass: Object.
	b := self stubClass: #FLStubB superclass: a.
	c := self stubClass: #FLStubC superclass: a.
	d := self stubClass: #FLStubD superclass: c.
	serializedClasses := {a. b. c. d}.
	
	materializedClasses := self resultOfSerializeRemoveAndMaterializeAll: serializedClasses.

	self assert: serializedClasses size equals: materializedClasses size.

	serializedClasses withIndexDo: [ :aClass :index | 
		self deny: aClass == (materializedClasses at: index) ].	

	self assert: Object equals: materializedClasses first superclass.
	self assert: materializedClasses first equals: materializedClasses second superclass.
	self assert: materializedClasses first equals: materializedClasses third superclass.
	self assert: materializedClasses third equals: materializedClasses fourth superclass.

	self assert: {materializedClasses second. materializedClasses third} asSet equals: materializedClasses first subclasses asSet.
	self assert: materializedClasses second subclasses isEmpty.
	self assert: {materializedClasses fourth} equals: materializedClasses third subclasses asArray.
	self assert: materializedClasses fourth subclasses isEmpty.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/22/2011 19:11'!
testCreateHierarchyWithExistingClasses
	"Tests materialization of a hierarchy of classes that are not defined in the image and classes that indeed are defined"
	
	| serializedClasses materializedClasses a b c d |	
	a := self stubClass: #FLStubA superclass: Object.
	b := self stubClass: #FLStubB superclass: a.
	c := self stubClass: #FLStubC superclass: a.
	d := self stubClass: #FLStubD superclass: Date.
	serializedClasses := {a. b. c. d}.
	
	materializedClasses := self resultOfSerializeRemoveAndMaterializeAll: serializedClasses.

	self assert: serializedClasses size equals: materializedClasses size.

	serializedClasses withIndexDo: [ :aClass :index | 
		self deny: aClass == (materializedClasses at: index) ].	

	self assert: Object equals: materializedClasses first superclass.
	self assert: Object == materializedClasses first superclass.
	self assert: materializedClasses first == materializedClasses second superclass.
	self assert: materializedClasses first == materializedClasses third superclass.
	"Notice that the superclass of FLStubD is Date and since it is present in the image, we need exaclty that instance. "
	self assert: Date == materializedClasses fourth superclass.


	self assert: {materializedClasses second. materializedClasses third} asSet equals: materializedClasses first subclasses asSet.
	self assert: materializedClasses second subclasses isEmpty.
	self assert: materializedClasses third subclasses isEmpty.
	self assert: materializedClasses fourth subclasses isEmpty.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/5/2011 15:24'!
testCreateHierarchyWithExistingClassesAndSubclasses
	"Tests materialization of a hierarchy of classes that are not defined in the image and classes that indeed are defined. In addition, the serialized class has instances that remain in the image (they are not serialized). This demonstrates that the instance variable 'subclasses' should be properly serialized and materialized"
	
	| serializedClasses materializedClasses a b |	
	a := self stubClass: #FLStubA superclass: Object.
	b := self stubClass: #FLStubB superclass: a.
	
	serializedClasses := {a.}.
	
	materializedClasses := self resultOfSerializeRemoveAndMaterializeAll: serializedClasses.

	self assert: serializedClasses size equals: materializedClasses size.

	serializedClasses withIndexDo: [ :aClass :index | 
		self deny: aClass == (materializedClasses at: index) ].	

	self assert: Object equals: materializedClasses first superclass.
	self assert: Object == materializedClasses first superclass.
	self deny: ( materializedClasses first subclasses isEmpty ).
	
	"This fail because b superclass is obsolete since FLStubA was removed, so we cannot really tested"	
	"self assert: b superclass equals: materializedClasses first.	"
	
	"We cannot test neither if they should be se same class or not:
	self assert: b superclass == materializedClasses first. "	
	
	

	! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 02:33'!
testCreateNormalClass
	"Tests materialization of a class that is not defined in the image"
	self testCorrectClassTypeFor: self stubClass! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/22/2011 19:08'!
testCreateVariableByteClass
	"Tests materialization of a variable byte class that is not defined in the image"
	self testCorrectClassTypeFor: self variableByteStubClass.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/22/2011 19:07'!
testCreateVariableClass
	"Tests materialization of a variable class that is not defined in the image"
	self testCorrectClassTypeFor: self variableStubClass.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/22/2011 19:07'!
testCreateVariableWordClass
	"Tests materialization of a variable word class that is not defined in the image"
	self testCorrectClassTypeFor: self variableWordStubClass.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/22/2011 19:07'!
testCreateWeakClass
	"Tests materialization of a weak class that is not defined in the image"
	self testCorrectClassTypeFor: self weakStubClass.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/5/2011 21:27'!
testCreateWithClassSideInitializeMethod
	"Tests materialization of a class-side #initialize method in a class or trait not defined in the image. It makes sure that Fuel doesn't call it automatically"

	| aClassOrTrait materializedClassOrTrait |
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait classSide compile: 'initialize self error: ''Fuel should not automatically call #initialize'' ' classified: #test.

	self shouldnt: [materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait] raise: Error.
	self should: [ (self newInstanceFrom: materializedClassOrTrait) class perform: #initialize] raise: Error.

	self assert: (materializedClassOrTrait classSide includesSelector: #initialize).
	self assert: #(initialize) asSet equals: materializedClassOrTrait classSide localSelectors.
	self assert: #test equals: (materializedClassOrTrait classSide whichCategoryIncludesSelector: #initialize).
	
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 4/11/2011 13:09'!
testCreateWithClassVariableAccessors
	"Tests materialization of a compiled method in a class not defined in the image. The class defines accessors for a class variable."
	
	| aClass materializedClass instance |	
	aClass := self stubClass.
	aClass addClassVarNamed: 'ClassVariable'.
	aClass compile: 'classVariable ^ClassVariable'.
	aClass compile: 'classVariable: value ClassVariable := value'.
	(self newInstanceFrom: aClass) classVariable: 42.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.
	instance := self newInstanceFrom: materializedClass.

	self assert: (materializedClass includesSelector: #classVariable).
	self assert: 42 equals: instance classVariable.
	instance classVariable: 0.
	self assert: 0 equals: instance classVariable.
! !

FLClassSerializationTest subclass: #FLExternalClassSerializationTest
	uses: FLTExternalClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLExternalClassSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of classes as *external* objects, i.e. the classes has to be present in the image at materialization time.!

FLExternalClassSerializationTest class
	uses: FLTExternalClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!

!FLExternalClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/27/2011 22:57'!
testMetaclass
	"Tests that the metaclass is correctly kept"
	
	| stream aPair resultPair |	

	aPair := (self stubClassWithInstanceVars: 'left right') new.
	stream := self serializationOf: aPair.
	resultPair := self materializationOn: stream reset.
	self assert: aPair class == resultPair class.! !

!FLExternalClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/11/2010 20:57'!
testVariableInsertion
	"Tests that serializer tolarates when there is a new instance variable on materialization"
	
	| stream aPair resultPair |	

	aPair := (self stubClassWithInstanceVars: 'left right') new.
	aPair instVarAt: 1 put: $A.	
	aPair instVarAt: 2 put: $B.	
	stream := self serializationOf: aPair.
	
	self stubClassWithInstanceVars: 'left middle right'.
	resultPair := self materializationOn: stream reset.
	self assert: $A equals: (resultPair instVarAt: 1).
	self assert: nil equals: (resultPair instVarAt: 2).	
	self assert: $B equals: (resultPair instVarAt: 3).	! !

!FLExternalClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/11/2010 20:01'!
testVariableOrderChange
	"Tests that serializer tolarates when the order in the instance variables changed between serialization and materialization"
	
	| pairClass stream aPair resultPair |	

	pairClass := self stubClassWithInstanceVars: 'left right'.
	aPair := pairClass new.
	aPair instVarAt: 1 put: $A.	
	aPair instVarAt: 2 put: $B.	
	stream := self serializationOf: aPair.
	
	pairClass := self stubClassWithInstanceVars: 'right left'.
	resultPair := self materializationOn: stream reset.
	self assert: $B equals: (resultPair instVarAt: 1).
	self assert: $A equals: (resultPair instVarAt: 2).	! !

!FLExternalClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/11/2010 20:58'!
testVariableRemoved
	"Tests that serializer tolarates when an instance variable is missing on materialization"
	
	| stream aPair resultPair |	

	aPair := (self stubClassWithInstanceVars: 'left right') new.
	aPair instVarAt: 1 put: $A.	
	aPair instVarAt: 2 put: $B.	
	stream := self serializationOf: aPair.
	
	self stubClassWithInstanceVars: 'right'.
	resultPair := self materializationOn: stream reset.
	self assert: $B equals: (resultPair instVarAt: 1).	! !

FLClassSerializationTest subclass: #FLModifyClassSerializationTest
	uses: FLTInternalSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLModifyClassSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire class, in the cases in which the class *is* present in the image at materialization time.!

FLModifyClassSerializationTest class
	uses: FLTInternalSerializationTest classTrait
	instanceVariableNames: ''!

!FLModifyClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 16:19'!
testModifyVariableAdd
	"Tests materialization of complete class that *IS* defined in the image"
	
	| aClass stream materializedClass variableNames |	
	"creates a class"
	aClass := self stubClassWithInstanceVars: 'aVariable'.
	variableNames := aClass instVarNames.
	
	"serialize it"
	stream := self internalSerializationOf: aClass.

	"modify it"
	self stubClassWithInstanceVars: ''.

	"materialize (and restore original version of variable names)"
	materializedClass := self materializationOn: stream reset.

	self assert: aClass == materializedClass.
	self assert: variableNames equals: aClass instVarNames.! !

!FLModifyClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 16:19'!
testModifyVariableRemove
	"Tests materialization of complete class that *IS* defined in the image"
	
	| aClass stream materializedClass variableNames |	
	"creates a class"
	aClass := self stubClassWithInstanceVars: ''.
	variableNames := aClass instVarNames.
	
	"serialize it"
	stream := self internalSerializationOf: aClass.

	"modify it"
	self stubClassWithInstanceVars: 'aVariable'.

	"materialize (and restore original version of variable names)"
	materializedClass := self materializationOn: stream reset.

	self assert: aClass == materializedClass.
	self assert: variableNames equals: aClass instVarNames.! !

FLSerializationTest subclass: #FLCompiledMethodSerializationTest
	instanceVariableNames: 'instanceVarForTesting'
	classVariableNames: 'ClassVariableForTesting'
	poolDictionaries: ''
	category: 'Fuel-Tests'!

FLCompiledMethodSerializationTest class
	instanceVariableNames: 'classInstanceVariableForTesting'!

FLCompiledMethodSerializationTest class
	instanceVariableNames: 'classInstanceVariableForTesting'!

!FLCompiledMethodSerializationTest classSide methodsFor: 'testing' stamp: 'MartinDias 10/7/2010 11:53'!
classInstanceVariableForTesting
	^ classInstanceVariableForTesting! !

!FLCompiledMethodSerializationTest classSide methodsFor: 'testing' stamp: 'MartinDias 10/7/2010 11:54'!
classInstanceVariableForTesting: anObject
	
	classInstanceVariableForTesting := anObject! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/18/2010 14:08'!
atInstanceVarForTestingPut: anObject
	
	instanceVarForTesting := anObject ! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/18/2010 14:08'!
classVariableForTesting
	
	^ ClassVariableForTesting! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/18/2010 14:08'!
classVariableForTesting: anObject
	
	ClassVariableForTesting := anObject ! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/18/2010 14:08'!
fortyTwo
	^ 42 ! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/18/2010 14:08'!
fortyTwoBlockClosure
	^ [ self fortyTwo ] ! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/20/2010 14:56'!
globalVariableForTesting

	^ Smalltalk globals at: #FLGlobalVariableForTesting ! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/20/2010 14:56'!
globalVariableForTesting: anObject

	 Smalltalk globals at: #FLGlobalVariableForTesting put: anObject! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2011 01:32'!
materializedCompiledMethod: aCompiledMethod

	| materializedCompiledMethod |
	materializedCompiledMethod := self resultOfSerializeAndMaterialize: aCompiledMethod.
	
	self flag: #todo. "See if this affirmation is still true: CompiledMethod>>= is too strict for us, so we check equality in this way"

	self deny: aCompiledMethod == materializedCompiledMethod.
	self assert: aCompiledMethod getSource equals: materializedCompiledMethod getSource.
	self assert: aCompiledMethod header equals: materializedCompiledMethod header.
	self assert: aCompiledMethod initialPC equals: materializedCompiledMethod initialPC.
	self assert: aCompiledMethod size equals: materializedCompiledMethod size.

	^ materializedCompiledMethod! !

!FLCompiledMethodSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/21/2010 13:53'!
tearDown

	super tearDown.
	Smalltalk globals removeKey: #FLGlobalVariableForTesting ifAbsent: []! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/11/2010 17:46'!
testBasicMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: Object >> #yourself.

	self assert: self == (materializedCompiledMethod valueWithReceiver: self arguments: #()).! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:22'!
testBlockClosureMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: self class >> #fortyTwoBlockClosure.

	self assert: 42 equals: (materializedCompiledMethod valueWithReceiver: self arguments: #()) value
! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:24'!
testClassInstanceVariableMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: self class class >> #classInstanceVariableForTesting:.

	self class classInstanceVariableForTesting: false.
	materializedCompiledMethod valueWithReceiver: self class arguments: { true }.
	self assert: self class classInstanceVariableForTesting.! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:31'!
testClassVariableMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: self class >> #classVariableForTesting:.

	self classVariableForTesting: false.
	materializedCompiledMethod valueWithReceiver: self arguments: { true }.
	self assert: self classVariableForTesting.! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:32'!
testGlobalVariableMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: self class >> #globalVariableForTesting:.
	
	self globalVariableForTesting: false.
	materializedCompiledMethod valueWithReceiver: self arguments: { true }.
	self assert: self globalVariableForTesting.! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/26/2010 14:57'!
testInstanceVariableMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: self class >> #atInstanceVarForTestingPut:.

	instanceVarForTesting := false.
	materializedCompiledMethod valueWithReceiver: self arguments: { true }.
	self assert: instanceVarForTesting.! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/22/2010 12:19'!
testNamedPrimitiveMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: Integer >> #bitAnd:.

	self assert: 16r30 equals: (materializedCompiledMethod valueWithReceiver: 16r3F arguments: #(16rF0)).! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:35'!
testPrimitiveMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: Object >> #perform:.

	self assert: 42 equals: (materializedCompiledMethod valueWithReceiver: self arguments: #(fortyTwo)).! !

!FLCompiledMethodSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 21:35'!
testSmallIntegerMethod

	| materializedCompiledMethod |

	materializedCompiledMethod := self materializedCompiledMethod: self class >> #fortyTwo.

	self assert: 42 equals: (materializedCompiledMethod valueWithReceiver: self arguments: #()).! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/11/2010 19:31'!
assertSerializationEqualityOf: anObject
	"Asserts that the original object and the materialized one are equal (but not the same)"

	| materialized |
	materialized := self resultOfSerializeAndMaterialize: anObject.
	self should: [ anObject ~~ materialized ].
	self assert: anObject equals: materialized! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/11/2010 19:31'!
assertSerializationIdentityOf: anObject
	"Asserts that the original object and the materialized one are the same"

	self should: [ anObject == (self resultOfSerializeAndMaterialize: anObject)  ]! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 16:22'!
classFactory
	^ classFactory! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 16:22'!
classFactory: anObject
	classFactory := anObject! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/25/2010 19:23'!
materializationOn: aStream
	"Returns the result of materializing aStream."

	^ (self materializerOn: aStream)
		materialize;
		root! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/25/2010 19:23'!
materializerOn: aStream

	^ FLMaterializer on: aStream! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/11/2010 19:31'!
resultOfSerializeAndMaterialize: anObject
	"Returns the result of serializing and materializing anObject."

	| stream |

	stream := self serializationOf: anObject.
	^ self materializationOn: stream reset


"	Using ReferenceStream:
	
	| stream |

	stream := ReferenceStream on: (RWBinaryOrTextStream on: '') binary.
	stream nextPut: anObject.

	stream := ReferenceStream on:  stream contents readStream.
	^ stream next"! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/11/2010 19:33'!
serializationOf: anObject
	"Returns the result of serializing anObject."

	^ (self serializer serialize: anObject) stream! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/11/2010 19:18'!
serializer

	^ FLSerializer on: self writeStream! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:20'!
streams
	^ streams! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:20'!
streams: anObject
	streams := anObject! !

FLSerializationTest subclass: #FLTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLTraitSerializationTest commentStamp: '<historical>' prior: 0!
I have the common behavior for testing trait serialization.!

FLTraitSerializationTest subclass: #FLCreateTraitSerialiationTest
	uses: FLTInternalSerializationTest + FLTCreateClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLCreateTraitSerialiationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire trait, in the cases in which it *isnot* present in the image at materialization time.!

FLCreateTraitSerialiationTest class
	uses: FLTInternalSerializationTest classTrait + FLTCreateClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!

FLTraitSerializationTest subclass: #FLExternalTraitSerializationTest
	uses: FLTExternalClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLExternalTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of traits as *external* objects, i.e. the traits has to be present in the image at materialization time.!

FLExternalTraitSerializationTest class
	uses: FLTExternalClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/23/2011 23:10'!
newClassOrTrait
	"Returns a trait for testing"

	^ Trait named: #FLStubMainTrait uses: {} category: self class category! !

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/24/2011 13:19'!
newInstanceFrom: aTrait

	| aClass |
	aClass := self stubClass.
	aClass addToComposition: aTrait.
	^ aClass new! !

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/24/2011 13:20'!
stubClass
	"Returns a class for testing"

	^ Object subclass: #FLStubSecondaryClass instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: self class category! !

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/23/2011 23:11'!
stubTrait
	"Returns a trait for testing"

	^ Trait named: #FLStubSecondaryTrait uses: {} category: self class category! !

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/23/2011 16:37'!
tearDown

	super tearDown.
	Smalltalk allClassesAndTraits select: [ :classOrTrait | classOrTrait name beginsWith: 'FLStub' ]
		thenDo: [ :classOrTrait | classOrTrait removeFromSystem ].

! !

!LargePositiveInteger methodsFor: '*Fuel' stamp: 'TristanBourgois 9/24/2010 14:21'!
fuelSerializer 

	^ FLLargePositiveIntegerClusterSerializer instance.! !

!ByteString methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:11'!
fuelAccept: aVisitor

	aVisitor visitPrimitive: self! !

!ByteString methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 13:39'!
fuelSerializer

	^ FLByteStringClusterSerializer instance! !

!MethodDictionary methodsFor: '*Fuel' stamp: 'MartinDias 3/22/2011 02:13'!
fuelSerializer 

	^ FLMethodDictionaryClusterSerializer instance! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 10/25/2010 15:23'!
fuelAccept: aVisitor

	aVisitor visitClassOrTrait: self! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 3/2/2011 23:31'!
fuelSerializer 

	^ FLClassClusterSerializer instance ! !

Object subclass: #FLAnalyzer
	instanceVariableNames: 'objectCount analyzers clusteredBaselevelObjects clusteredExternalMetalevelObjects clusteredInternalMetalevelObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Analyzer'!

!FLAnalyzer commentStamp: '<historical>' prior: 0!
I am responsible of traverse the graph of references starting on a collection of root objects. I will produce dictionaries that associate each traversed object to a serializer.

An example of use is

	FLAnalyzer newDefault 
		analyzeAll: { 1@2. 3@4 };
		clusteredInstances.
	
The results are used by FLSerializer to serialize the analyzed objects into a stream.!

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 10/26/2010 10:32'!
new

	self error: 'Should use an specific instance creation message.'! !

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 10/26/2010 10:31'!
newDefault

	^ self basicNew 
		initialize; 
		initializeDefaultAnalyzers;
		yourself! !

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 3/26/2011 17:13'!
newDefaultInternalObjects: aCollection
	"Returns an instance with default configuration, declaring an internal the objects in the collection."

	| analyzer globalAnalyzer classPoolAssociationAnalyzer |
	analyzer := self basicNew 
		initialize; 
		yourself.
		
	self flag: #todo. "Reify configuration and remove this crap"
	
	globalAnalyzer := (FLGlobalAnalyzer newWith: analyzer) 
		notExternalObjects: aCollection; 
		yourself.
		
	classPoolAssociationAnalyzer := (FLClassPoolAssociationAnalyzer newWith: analyzer) 
		notExternalObjects: aCollection; 
		yourself.
		
	analyzer analyzers 
		add: (FLGlobalAssociationAnalyzer newWith: analyzer);
		add: classPoolAssociationAnalyzer;
		add: globalAnalyzer;
		add: (FLDefaultAnalyzer newWith: analyzer).
		
	^ analyzer! !

!FLAnalyzer methodsFor: 'analyze' stamp: 'MartinDias 10/25/2010 15:07'!
analyze: anObject

	(self analyzerFor: anObject) value: anObject ! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 10/26/2010 10:35'!
analyzerFor: anObject

	^ analyzers detect: [ :anAnalyzer | anAnalyzer analyzes: anObject ].
! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 21:05'!
analyzers
	^ analyzers! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 14:58'!
clusteredBaselevelObjects
	
	^ clusteredBaselevelObjects! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 23:57'!
clusteredExternalMetalevelObjects
	
	^ clusteredExternalMetalevelObjects ! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 23:57'!
clusteredInternalMetalevelObjects
	
	^ clusteredInternalMetalevelObjects ! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 23:59'!
clusteredObjects
	"Returns a new collection with all the clustered objects, in Association instances."
	
	^ (OrderedCollection new: 
			clusteredExternalMetalevelObjects size + 
			clusteredInternalMetalevelObjects size + 
			clusteredBaselevelObjects size) 
		addAll: clusteredExternalMetalevelObjects associations;
		addAll: clusteredInternalMetalevelObjects associations;
		addAll: clusteredBaselevelObjects associations;
		yourself.! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 11/11/2010 16:11'!
clusters: aDictionary add: anObject

	self
		clusters: aDictionary
		add: anObject
		serializer: anObject fuelSerializer! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 11/11/2010 16:19'!
clusters: aDictionary add: anObject serializer: aSerializer

	self 
		clusters: aDictionary 
		add: anObject 
		serializer: aSerializer 
		collectionFactoryBlock: [ IdentitySet new ]
! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 11/11/2010 18:31'!
clusters: aDictionary add: anObject serializer: aSerializer collectionFactoryBlock: aBlockForNewCollection

	| clusterObjects |
	clusterObjects := aDictionary at: aSerializer ifAbsentPut: [ aBlockForNewCollection value ].
	(clusterObjects includes: anObject)
		ifTrue: [ ^ self ].
	clusterObjects add: anObject.
	objectCount := objectCount + 1.
	aSerializer referencesOf: anObject do: [ :aChild | self analyze: aChild ]! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 3/2/2011 23:58'!
initialize

	clusteredBaselevelObjects := Dictionary new.
	clusteredExternalMetalevelObjects := Dictionary new.
	clusteredInternalMetalevelObjects := Dictionary new.
	objectCount := 0.
	analyzers := OrderedCollection new.
! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 10/26/2010 10:30'!
initializeDefaultAnalyzers

	analyzers 
		add: (FLGlobalAssociationAnalyzer newWith: self);
		add: (FLClassPoolAssociationAnalyzer newWith: self);
		add: (FLGlobalAnalyzer newWith: self);
		add: (FLDefaultAnalyzer newWith: self)
! !

!FLAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 10/19/2010 10:31'!
objectCount

	^ objectCount! !

Object subclass: #FLClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

FLClusterSerializer class
	instanceVariableNames: 'instance serializersById'!

!FLClusterSerializer commentStamp: '<historical>' prior: 0!
I represent a special way of serializing and materializing some specific kind of objects.

Each of my non-abstract subclasses answers to #id with an unique integer, that identifies it. 

Sending me the #forId: message you obtain an instance that corresponds with an specific id. 

Every object responds to #fuelSerializer with its corresponding subclass of me. For example:

	42 fuelSerializer 
	
will return a serializer for that specific instance of SmallInteger.!

FLClusterSerializer class
	instanceVariableNames: 'instance serializersById'!

!FLClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 15:24'!
allSubclassesWithId
	"Answers a collection with all the subclasses that define an id. In other words, all the non-abstract subclasses are returned"

	^self allSubclasses select: [ :aClass | 
			[ aClass id isInteger ] ifError: [ false ] ] 
	! !

!FLClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 10/18/2010 12:34'!
forId: anInteger
	"Returns the serializer instance that corresponds to the id received"

	^ self serializersById 
			at: anInteger 
			ifAbsent: [ self error: 'Could not find a serializer with id=', anInteger asString ].
	
! !

!FLClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:56'!
id
	"Answer an integer identifier. It must be between 0 and 255. 
	Each type of primitive object should have a different one. 
	validateIdDistribution checks that there is no overlap."
	
	self subclassResponsibility ! !

!FLClusterSerializer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/21/2011 17:34'!
initialize
	"Initializes singleton class variables"

	super initialize.
	self resetHierarchy.! !

!FLClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 10/15/2010 11:29'!
instance
	"Returns the singleton instance of this class. It may not be sent to abstract classes, see class doc."
	
	^ instance ifNil: [ instance := self basicNew initialize ]
	
! !

!FLClusterSerializer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/21/2011 17:33'!
reset
	"Resets singleton class variables"

	instance := nil.
	serializersById := nil.! !

!FLClusterSerializer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/21/2011 17:34'!
resetHierarchy
	"Resets class variables in the whole hierarchy"

	self withAllSubclasses do: [ :x | x reset ]! !

!FLClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 15:16'!
serializersById
	"Returns a dictionary that indexes the serializers by their ids"

	serializersById ifNil: [
			serializersById := IdentityDictionary new.
			self allSubclassesWithId 
				do: [ :aSerializerClass | 
					serializersById 
						at: aSerializerClass id 
						put: aSerializerClass instance ] ].
	^ serializersById 
! !

!FLClusterSerializer classSide methodsFor: 'checking' stamp: 'MartinDias 10/12/2010 16:06'!
validateIdDistribution
	"Verifies that subclasses do not have overlapping ids"
	
	self assert: (self allSubclassesWithId collect: [ :aClass | aClass id ]) hasAnyDuplicate not
		description: 'Overlapping id detected in FLClusterSerializer subclasses'! !

!FLClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 11:35'!
materializeAllWith: aMaterializer

	self subclassResponsibility ! !

!FLClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 3/2/2011 23:07'!
referencesOf: anObject do: aBlock
	"Evaluates the received block with each object referenced by anObject"! !

!FLClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 11:11'!
serializeAll: clusterInstances with: aSerializer

	self subclassResponsibility ! !

FLClusterSerializer subclass: #FLIteratingClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLIteratingClusterSerializer commentStamp: '<historical>' prior: 0!
I am a template class whose algorithm for serialize a collection of objects consists on delegate the serialization of each individual object to the subclass.!

FLIteratingClusterSerializer subclass: #FLClassOrTraitClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLClassOrTraitClusterSerializer commentStamp: '<historical>' prior: 0!
I have the common behavior for serializing a class or a trait with full detail. The class or trait will be recreated completely on the materializing image.!

FLClassOrTraitClusterSerializer subclass: #FLClassClusterSerializer
	instanceVariableNames: ''
	classVariableNames: 'SubclassSelectors'
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLClassClusterSerializer commentStamp: '<historical>' prior: 0!
I serialize a class with full detail. The class will be recreated completely on the materializing image.

TODO: describe different scenarios for materialization:

- when the class already exists in the image:
-- the class' format changes (for example, #isVariable changes its value)
-- the instance variables are different

- when the superclass doesn't exist

- other!

!FLClassClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 23:09'!
id
	^ 17! !

!FLClassClusterSerializer classSide methodsFor: 'initialize-release' stamp: 'MartinDias 3/29/2011 22:35'!
initialize

	SubclassSelectors := Dictionary new
		at: #normal put: #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #weak put: #weakSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #variable put: #variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #bytes put: #variableByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #words put: #variableWordSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		at: #compiledMethod put: #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:;
		yourself.! !

!FLClassClusterSerializer methodsFor: 'private' stamp: 'MartinDias 3/18/2011 15:58'!
hackedClassPool: aClass

	self flag: #todo. "fix reifying better the graph iteration"
	^ aClass classPool isEmpty ifTrue: [ nil ] ifFalse: [ aClass classPool ]
	! !

!FLClassClusterSerializer methodsFor: 'private' stamp: 'MartinDias 3/20/2011 14:44'!
hackedSharedPools: aClass

	self flag: #todo. "fix reifying better the graph iteration"
	^ aClass sharedPools isEmpty ifTrue: [ nil ] ifFalse: [ aClass sharedPools ]
	! !

!FLClassClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 22:42'!
materializeInstanceWith: aMaterializer

	| aClass superclass category name instanceVariableNames classSideInstanceVariableNames subclassSelector |
	superclass := aMaterializer nextReferencedObject.
	category := aMaterializer stream nextString.
	name := aMaterializer stream nextString asSymbol.
	instanceVariableNames := aMaterializer stream nextString.
	classSideInstanceVariableNames := aMaterializer stream nextString.
	subclassSelector := SubclassSelectors at: aMaterializer stream nextString asSymbol.

	aClass := superclass perform: subclassSelector withArguments: {name. instanceVariableNames. ''. ''. category}.
	aClass class instanceVariableNames: classSideInstanceVariableNames.
	^aClass! !

!FLClassClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 22:28'!
materializeReferencesOf: aClass with: aMaterializer

	super materializeReferencesOf: aClass with: aMaterializer.

	aClass classPool: aMaterializer nextReferencedObject.
	aClass sharedPools: aMaterializer nextReferencedObject.

	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aClass instVarAt: index put: aMaterializer nextReferencedObject ]! !

!FLClassClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 3/29/2011 22:26'!
referencesOf: aClass do: aBlock

	super referencesOf: aClass do: aBlock.
	
	aBlock value: aClass superclass.
	aBlock value: (self hackedClassPool: aClass).
	aBlock value: (self hackedSharedPools: aClass). 

	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aBlock value: (aClass instVarAt: index) ]! !

!FLClassClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:37'!
serializeInstance: aClass with: aSerializer

	aSerializer nextPutReferenceTo: aClass superclass.
	aSerializer stream nextStringPut: aClass basicCategory.
	aSerializer stream nextStringPut: aClass name asString.
	aSerializer stream nextStringPut: aClass instanceVariablesString.
	aSerializer stream nextStringPut: aClass class instanceVariablesString.
	aSerializer stream nextStringPut: aClass typeOfClass asString.! !

!FLClassClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 22:30'!
serializeReferencesOf: aClass with: aSerializer

	super serializeReferencesOf: aClass with: aSerializer.

	aSerializer nextPutReferenceTo: (self hackedClassPool: aClass). 
	aSerializer nextPutReferenceTo: (self hackedSharedPools: aClass). 

	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aSerializer nextPutReferenceTo: (aClass instVarAt: index) ]! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 22:14'!
materializeInstanceWith: aMaterializer
	
	self explicitRequirement! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:47'!
materializePostReferencesOf: aClassOrTrait with: aMaterializer

	aClassOrTrait initializeTraitComposition: aMaterializer nextReferencedObject.
	aClassOrTrait classSide initializeTraitComposition: aMaterializer nextReferencedObject.! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:47'!
materializeReferencesOf: aClassOrTrait with: aMaterializer

	aClassOrTrait methodDictionary: aMaterializer nextReferencedObject.
	aClassOrTrait organization: aMaterializer nextReferencedObject.
	aClassOrTrait basicLocalSelectors: aMaterializer nextReferencedObject.
	
	aClassOrTrait classSide methodDictionary: aMaterializer nextReferencedObject.
	aClassOrTrait classSide organization: aMaterializer nextReferencedObject.
	aClassOrTrait classSide basicLocalSelectors: aMaterializer nextReferencedObject.! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:37'!
materializeWith: aMaterializer

	| classOrTrait |
	classOrTrait := self materializeInstanceWith: aMaterializer.
	
	aMaterializer referenceMaterializingActionsAdd: [ 
		self materializeReferencesOf: classOrTrait with: aMaterializer ].

	aMaterializer postReferenceMaterializingActionsAdd: [ 
		self materializePostReferencesOf: classOrTrait with: aMaterializer ].

	^classOrTrait! !

!FLClassOrTraitClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 3/29/2011 23:48'!
referencesOf: aClassOrTrait do: aBlock

	aBlock value: aClassOrTrait methodDictionary.
	aBlock value: aClassOrTrait organization.
	aBlock value: aClassOrTrait traitComposition.
	aBlock value: aClassOrTrait basicLocalSelectors.
	
	aBlock value: aClassOrTrait classSide methodDictionary.
	aBlock value: aClassOrTrait classSide organization.
	aBlock value: aClassOrTrait classSide traitComposition.
	aBlock value: aClassOrTrait classSide basicLocalSelectors.! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:37'!
serialize: aClassOrTrait with: aSerializer

	self serializeInstance: aClassOrTrait with: aSerializer.

	aSerializer referenceSerializingActionsAdd:
		[ self serializeReferencesOf: aClassOrTrait with: aSerializer ].

	aSerializer postReferenceSerializingActionsAdd:
		[ self serializePostReferencesOf: aClassOrTrait with: aSerializer ].
! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 22:17'!
serializeInstance: aClassOrTrait with: aSerializer

	self explicitRequirement ! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:48'!
serializePostReferencesOf: aClassOrTrait with: aSerializer

	aSerializer nextPutReferenceTo: aClassOrTrait traitComposition.
	aSerializer nextPutReferenceTo: aClassOrTrait classSide traitComposition.
! !

!FLClassOrTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:48'!
serializeReferencesOf: aClassOrTrait with: aSerializer

	aSerializer nextPutReferenceTo: aClassOrTrait methodDictionary.
	aSerializer nextPutReferenceTo: aClassOrTrait organization. 
	aSerializer nextPutReferenceTo: aClassOrTrait basicLocalSelectors.
	
	aSerializer nextPutReferenceTo: aClassOrTrait classSide methodDictionary.
	aSerializer nextPutReferenceTo: aClassOrTrait classSide organization.
	aSerializer nextPutReferenceTo: aClassOrTrait classSide basicLocalSelectors.! !

FLClassOrTraitClusterSerializer subclass: #FLTraitClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLTraitClusterSerializer commentStamp: '<historical>' prior: 0!
I serialize a trait with full detail. The trait will be recreated completely on the materializing image.!

!FLTraitClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/24/2011 03:35'!
id
	^ 18! !

!FLTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:40'!
materializeInstanceWith: aMaterializer

	| category name |
	category := aMaterializer stream nextString.
	name := aMaterializer stream nextString asSymbol.
	^Trait named: name uses: {} category: category.! !

!FLTraitClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/29/2011 23:39'!
serializeInstance: aTrait with: aSerializer

	aSerializer stream nextStringPut: aTrait basicCategory.
	aSerializer stream nextStringPut: aTrait name asString.! !

FLIteratingClusterSerializer subclass: #FLClassPoolAssociationClusterSerializer
	instanceVariableNames: 'ownerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLClassPoolAssociationClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for class variable associations, i.e. bindings that belong to the class pool of a Class included on Smalltalk globals.!

!FLClassPoolAssociationClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/18/2010 18:02'!
id

	^ 20! !

!FLClassPoolAssociationClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 10/18/2010 17:55'!
newFor: aClass

	^ self basicNew
		initialize;
		ownerClass: aClass;
		yourself.! !

!FLClassPoolAssociationClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/26/2011 17:04'!
materializeWith: aMaterializer

	| keyName |
	ownerClass := aMaterializer nextReferencedObject.
	keyName := aMaterializer stream nextString.
	^ ownerClass classPool associationAt: keyName asSymbol ! !

!FLClassPoolAssociationClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 10/18/2010 17:59'!
ownerClass: anObject
	ownerClass := anObject! !

!FLClassPoolAssociationClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 11/11/2010 18:31'!
referencesOf: anAssociation do: aBlock

	aBlock value: ownerClass! !

!FLClassPoolAssociationClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/18/2010 18:07'!
serialize: anAssociation with: aSerializer

	aSerializer nextPutReferenceTo: ownerClass.
	aSerializer stream nextStringPut: anAssociation key asString! !

FLIteratingClusterSerializer subclass: #FLClassSideClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLClassSideClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for Metaclasses and ClassTrait.!

!FLClassSideClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:24'!
id
	^ 12! !

!FLClassSideClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/24/2011 12:27'!
materializeWith: aMaterializer

	^ aMaterializer nextReferencedObject classSide! !

!FLClassSideClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 3/23/2011 18:06'!
referencesOf: aClassOrTraitClassSide do: aBlock

	aBlock value: aClassOrTraitClassSide instanceSide! !

!FLClassSideClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/24/2011 12:27'!
serialize: aClassOrTraitClassSide with: aSerializer

	aSerializer nextPutReferenceTo: aClassOrTraitClassSide instanceSide! !

FLIteratingClusterSerializer subclass: #FLCompiledMethodClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLCompiledMethodClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for CompiledMethod instances.!

!FLCompiledMethodClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:19'!
id
	^2! !

!FLCompiledMethodClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 10/7/2010 00:36'!
materializeLiteralsTo: compiledMethod with: aMaterializer

	1 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterializer nextReferencedObject ]
! !

!FLCompiledMethodClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/21/2011 17:41'!
materializeWith: aMaterializer

	| compiledMethod header source trailer bytecodesSize aReadStream |

	aReadStream := aMaterializer stream.

	header := (aReadStream nextWord << 16) bitOr: aReadStream nextWord.
	bytecodesSize := aReadStream nextWord.
	source := FLRemoteStringClusterSerializer configuredInstance materializeFrom: aReadStream.

	trailer := CompiledMethodTrailer new 
		sourcePointer: source sourcePointer;
		yourself.

	compiledMethod := trailer createMethod: bytecodesSize header: header.

	compiledMethod initialPC - 1 to: compiledMethod endPC
		do: [ :index | compiledMethod at: index put: aReadStream next ].

	aMaterializer referenceMaterializingActionsAdd: 
		[ self materializeLiteralsTo: compiledMethod with: aMaterializer ].
		
	^ compiledMethod! !

!FLCompiledMethodClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 3/26/2011 16:27'!
referencesOf: anObject do: aBlock

	1 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLCompiledMethodClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/21/2011 17:41'!
serialize: aCompiledMethod with: aSerializer

	| header source aWriteStream bytecodesSize |

	aWriteStream := aSerializer stream.

	header := aCompiledMethod header.
	bytecodesSize := aCompiledMethod endPC - aCompiledMethod initialPC + 1.
	source := RemoteString new setSourcePointer: aCompiledMethod sourcePointer.

	aWriteStream 
		nextWordPut: (header >> 16);
		nextWordPut: header;
		nextWordPut: bytecodesSize.

	FLRemoteStringClusterSerializer configuredInstance serialize: source on: aWriteStream.
	
	aCompiledMethod initialPC - 1 to: aCompiledMethod endPC
		do: [ :index | aWriteStream nextPut: (aCompiledMethod at: index) ].

	aSerializer referenceSerializingActionsAdd: 
		[ self serializeLiteralsOf: aCompiledMethod with: aSerializer ]! !

!FLCompiledMethodClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 10/7/2010 00:36'!
serializeLiteralsOf: aCompiledMethod with: aSerializer

	1 to: aCompiledMethod numLiterals 
		do: [ :index | aSerializer nextPutReferenceTo: (aCompiledMethod literalAt: index) ]! !

FLCompiledMethodClusterSerializer subclass: #FLNamedPrimitiveMethodClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLNamedPrimitiveMethodClusterSerializer commentStamp: 'MartinDias 11/2/2010 15:33' prior: 0!
I am a serializer for CompiledMethods which are <primitive:module:> primitives.

For example, Integer >> bitAnd: should be serialized with me.!

!FLNamedPrimitiveMethodClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/22/2010 12:08'!
id

	^ 23! !

!FLNamedPrimitiveMethodClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 11/11/2010 18:33'!
materializeLiteralsTo: compiledMethod with: aMaterializer

	| module name |
	module := aMaterializer nextReferencedObject.
	name := aMaterializer nextReferencedObject.
	compiledMethod literalAt: 1 put: (Array with: module with: name with: 0 with: 0).

	2 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterializer nextReferencedObject ]
! !

!FLNamedPrimitiveMethodClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 11/11/2010 18:31'!
referencesOf: anObject do: aBlock

	aBlock 
		value: (anObject literalAt: 1) first;
		value: (anObject literalAt: 1) second. 

	2 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLNamedPrimitiveMethodClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 10/26/2010 14:43'!
serializeLiteralsOf: aCompiledMethod with: aSerializer

	| primitiveArguments |
	
	primitiveArguments := aCompiledMethod literalAt: 1.
	aSerializer 
		nextPutReferenceTo: primitiveArguments first;
		nextPutReferenceTo: primitiveArguments second.

	2 to: aCompiledMethod numLiterals 
		do: [ :index | aSerializer nextPutReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLIteratingClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:27'!
materializeAllWith: aMaterializer
	"Template method that let to the subclass the materialization using the received materializer"

	| instanceCount |
	instanceCount := aMaterializer nextInteger.

	instanceCount timesRepeat: 
		[ aMaterializer addMaterialized: 
			(self materializeWith: aMaterializer) ].! !

!FLIteratingClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:30'!
materializeWith: aMaterializer
	"Hook method that each subclass should customize for its special way of materializing"

	^ self subclassResponsibility! !

!FLIteratingClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/11/2010 18:02'!
serialize: anObject with: aSerializer
	"Hook method that each subclass should customize for its special way of serializing"

	self subclassResponsibility ! !

!FLIteratingClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:27'!
serializeAll: aCollection with: aSerializer
	"Template method that let to the subclass the serialization of each instance received"

	aSerializer nextPutInteger: aCollection size.
	aCollection do: [ :instance | self serialize: instance with: aSerializer ]! !

FLIteratingClusterSerializer subclass: #FLMethodDictionaryClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLMethodDictionaryClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/22/2011 01:08'!
id
	^ 96! !

!FLMethodDictionaryClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/22/2011 02:11'!
materializeKeysAndValuesTo: aDictionary with: aMaterializer size: size
	
	size timesRepeat: [
		| method |
		method := aMaterializer nextReferencedObject.
		aDictionary at: method selector put: method ]! !

!FLMethodDictionaryClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/24/2011 12:57'!
materializeWith: aMaterializer
	
	| dictionary size |
	self flag: #todo. "Find a wider solution for collections with hashing problems"
	
	size := aMaterializer nextInteger.
	dictionary := MethodDictionary new: size.
	
	aMaterializer referenceMaterializingActionsAdd: [
		self materializeKeysAndValuesTo: dictionary with: aMaterializer size: size ].
	
	^ dictionary! !

!FLMethodDictionaryClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 3/22/2011 02:06'!
referencesOf: aDictionary do: aBlock

	aDictionary do: [ :value | aBlock value: value ]! !

!FLMethodDictionaryClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/22/2011 02:03'!
serialize: aDictionary with: aSerializer

	aSerializer nextPutInteger: aDictionary size.
	aSerializer referenceSerializingActionsAdd: [
		self serializeKeysAndValuesOf: aDictionary with: aSerializer ]! !

!FLMethodDictionaryClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/24/2011 12:57'!
serializeKeysAndValuesOf: aDictionary with: aSerializer

	aDictionary do: [ :value | aSerializer nextPutReferenceTo: value ]! !

FLClusterSerializer subclass: #FLObjectClusterSerializer
	uses: FLTClassParameter
	instanceVariableNames: 'parameterClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLObjectClusterSerializer commentStamp: '<historical>' prior: 0!
I implement a generic way of serializing and materializing an object, for those that do not need more special way of doing it.

For example,

	10@20 fuelSerializer 

returns an instance of me.

!

FLObjectClusterSerializer class
	uses: FLTClassParameter classTrait
	instanceVariableNames: ''!

!FLObjectClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:24'!
id

	^0! !

!FLObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 2/26/2011 03:07'!
materializeAllWith: aMaterializer

	| instanceCount clusterInstances class instanceVariablesMapping numberOfInstanceVariables instanceVariableNames |

	self flag: #todo. "refactor this big method"

	class := aMaterializer nextReferencedObject.
	numberOfInstanceVariables := aMaterializer stream next.
	instanceVariableNames := (1 to: numberOfInstanceVariables) collect: [ :index | aMaterializer stream nextString ].
	instanceCount := aMaterializer nextInteger.
	clusterInstances := OrderedCollection new: instanceCount.

	instanceVariablesMapping := FLInstanceVariablesMapping 
			currentClass: class 
			originalInstanceVariableNames: instanceVariableNames.

	class isVariable
		ifTrue: [ 
			instanceCount timesRepeat: [ clusterInstances add: (class basicNew: aMaterializer nextInteger) ].
			aMaterializer referenceMaterializingActionsAdd: [ 
				self materializeVariableReferencesOfAll: clusterInstances with: aMaterializer ] ]
		ifFalse: [ 
			instanceCount timesRepeat: [ clusterInstances add: class basicNew ] ].

	class hasInstanceVariables
		ifTrue: [ 
			aMaterializer referenceMaterializingActionsAdd: [ 
				self 
					materializeReferencesOfAll: clusterInstances 
					with: aMaterializer 
					instanceVariablesMapping: instanceVariablesMapping ] ].
		
	aMaterializer addAllMaterialized: clusterInstances! !

!FLObjectClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 11/12/2010 01:03'!
materializeReferencesOfAll: instances with: aMaterializer instanceVariablesMapping: instanceVariablesMapping
	
	instances
		do: [ :instance |
			instanceVariablesMapping materializeReferencesOf: instance with: aMaterializer ]! !

!FLObjectClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 11/11/2010 18:14'!
materializeVariableReferencesOfAll: instances with: aMaterializer

	instances
		do: [ :instance |
			1 to: instance basicSize 
				do: [ :index | instance basicAt: index put: aMaterializer nextReferencedObject ]]! !

!FLObjectClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 4/19/2011 12:39'!
parameterClass

	^ parameterClass ! !

!FLObjectClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 4/19/2011 12:39'!
parameterClass: aClass

	parameterClass:= aClass ! !

!FLObjectClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 11/11/2010 18:31'!
referencesOf: anObject do: aBlock

	aBlock value: anObject class.

	anObject class isVariable 
		ifTrue: [ 
			1 to: anObject basicSize 
				do: [ :index | aBlock value: (anObject basicAt: index) ] ].
	
	1 to: anObject class instSize 
		do: [ :index | aBlock value: (anObject instVarAt: index) ]! !

!FLObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 4/19/2011 12:39'!
serializeAll: clusterInstances with: aSerializer

	self serializeClassInfoWith: aSerializer.
	aSerializer nextPutInteger: clusterInstances size.

	parameterClass isVariable
		ifTrue: [ 
			clusterInstances do: [ :instance | aSerializer nextPutInteger: instance basicSize ].
			aSerializer referenceSerializingActionsAdd:
				[ self serializeVariableReferencesOfAll: clusterInstances with: aSerializer ] ].

	parameterClass hasInstanceVariables
		ifTrue: [ 
			aSerializer referenceSerializingActionsAdd:
				[ self serializeReferencesOfAll: clusterInstances with: aSerializer ] ]! !

!FLObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 4/19/2011 12:39'!
serializeClassInfoWith: aSerializer

	aSerializer nextPutReferenceTo: parameterClass.
	aSerializer stream nextPut: parameterClass instSize. 
	parameterClass instVarNamesAndOffsetsDo: [ :name :index | aSerializer stream nextStringPut: name ].
	
	self flag: #todo. "only 256 instance variables?"
! !

!FLObjectClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 4/19/2011 12:39'!
serializeReferencesOfAll: instances with: aSerializer 
	
	instances 
		do: [ :anInstance |
			1 to: parameterClass instSize do: [ :index |
				aSerializer nextPutReferenceTo: (anInstance instVarAt: index) ]]
				
	! !

!FLObjectClusterSerializer methodsFor: 'protected' stamp: 'MartinDias 10/13/2010 11:14'!
serializeVariableReferencesOfAll: instances with: aSerializer

	instances 
		do: [ :anInstance |
			1 to: anInstance basicSize do: [ :index |
				aSerializer nextPutReferenceTo: (anInstance basicAt: index) ]]! !

FLClusterSerializer subclass: #FLPrimitiveClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLPrimitiveClusterSerializer commentStamp: '<historical>' prior: 0!
I am a template class whose algorithm for serialize a collection of objects is to delegate the serialization of each individual object to the subclass. 

(TODO) My main difference with FLIteratingClusterSerializer is that my method hooks only receive a stream and not the complete instance of FLSerializer, because they have not need. But possibly there is no reason for being different classes.!

FLPrimitiveClusterSerializer subclass: #FLByteArrayClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLByteArrayClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for ByteArray.!

!FLByteArrayClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/25/2010 14:37'!
id

	^25! !

!FLByteArrayClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/25/2010 14:37'!
materializeFrom: aReadStream

	^ aReadStream nextString asByteArray! !

!FLByteArrayClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/25/2010 14:37'!
serialize: aByteArray on: aWriteStream

	aWriteStream nextStringPut: aByteArray asString ! !

FLPrimitiveClusterSerializer subclass: #FLByteStringClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLByteStringClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for ByteString.!

!FLByteStringClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/25/2010 13:32'!
id

	^24! !

!FLByteStringClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/25/2010 13:38'!
materializeFrom: aReadStream

	^ aReadStream nextString! !

!FLByteStringClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/25/2010 13:38'!
serialize: aString on: aWriteStream

	aWriteStream nextStringPut: aString! !

FLPrimitiveClusterSerializer subclass: #FLCharacterClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLCharacterClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for Character.!

!FLCharacterClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:24'!
id

	^1! !

!FLCharacterClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 11:06'!
materializeFrom: aReadStream

	^Character value: aReadStream next! !

!FLCharacterClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/22/2010 17:33'!
serialize: aCharacter on: aWriteStream

	aWriteStream nextPut: aCharacter codePoint! !

FLPrimitiveClusterSerializer subclass: #FLFloatClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLFloatClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for 8-byte Floats. 

Implementation took from ReferenceStream.
!

!FLFloatClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/20/2010 22:45'!
id
	^21! !

!FLFloatClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/21/2010 13:27'!
materializeFrom: aReadStream

	^ (Float new: 2)
		at: 1 put: (aReadStream nextNumber: 4);
		at: 2 put: (aReadStream nextNumber: 4);
		yourself.
! !

!FLFloatClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/21/2010 10:39'!
serialize: aFloat on: aWriteStream

	aWriteStream 
		nextNumber: 4 put: (aFloat at: 1);
		nextNumber: 4 put: (aFloat at: 2).! !

FLPrimitiveClusterSerializer subclass: #FLGlobalAssociationClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLGlobalAssociationClusterSerializer commentStamp: '<historical>' prior: 0!
I serialize variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
could be serialized by me.!

!FLGlobalAssociationClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/18/2010 12:16'!
id
	^ 19! !

!FLGlobalAssociationClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 15:08'!
materializeFrom: aReadStream

	^ Smalltalk globals 
		associationAt: aReadStream nextString asSymbol
		ifAbsent: [ self error: 'Association not found.' ] ! !

!FLGlobalAssociationClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/18/2010 12:10'!
serialize: anAssociation on: aWriteStream
	
	aWriteStream nextStringPut: anAssociation key! !

FLPrimitiveClusterSerializer subclass: #FLGlobalClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLGlobalClusterSerializer commentStamp: 'MartinDias 11/15/2010 11:16' prior: 0!
I am a serializer for known objects that belong to Smalltalk global dictionary.

For example, Transcript could be serialized by me.!

!FLGlobalClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:25'!
id
	^ 11! !

!FLGlobalClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 14:52'!
materializeFrom: aReadStream

	^ Smalltalk globals 
		at: aReadStream nextString asSymbol 
		ifAbsent: [ self error: 'Smalltalk global not found' ]! !

!FLGlobalClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 14:52'!
serialize: anObject on: aWriteStream
	
	aWriteStream 
		nextStringPut: (Smalltalk globals keyAtValue: anObject) asString.! !

FLPrimitiveClusterSerializer subclass: #FLLargeNegativeIntegerClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLLargeNegativeIntegerClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for LargeNegativeInteger.!

!FLLargeNegativeIntegerClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:25'!
id
	^8! !

!FLLargeNegativeIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 21:39'!
materializeFrom: aReadStream

	| size value |
	size := aReadStream next.
	value := 0.
	size timesRepeat: [value := (value bitShift: 8) bitOr: aReadStream next].
	^ value negated ! !

!FLLargeNegativeIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 10:44'!
serialize: anInteger on: aWriteStream

	| size byte positive |
	positive := anInteger abs.
	size := (positive log: 256) floor + 1.
	aWriteStream nextPut:size.
	(size - 1 to: 0 by: -1) do:
		[:index |
			byte := (positive bitShift: index * -8) bitAnd: 255.
			aWriteStream nextPut: byte]! !

FLPrimitiveClusterSerializer subclass: #FLLargePositiveIntegerClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLLargePositiveIntegerClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for LargePositiveInteger.!

!FLLargePositiveIntegerClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:25'!
id
	^7! !

!FLLargePositiveIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 21:39'!
materializeFrom: aReadStream

	| size value |
	size:=aReadStream next.
	value := 0.
	size timesRepeat: [value := (value bitShift: 8) bitOr: aReadStream next].
	^ value! !

!FLLargePositiveIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/27/2010 13:42'!
serialize: anInteger on: aWriteStream

	| size byte |
	size:=(anInteger log: 256) floor + 1.
	aWriteStream nextPut: size.
	(size - 1 to: 0 by: -1) do:
		[:index |
		byte := (anInteger bitShift: index * -8) bitAnd: 255.
		aWriteStream nextPut: byte]! !

FLPrimitiveClusterSerializer subclass: #FLNilTrueFalseClusterSerializer
	instanceVariableNames: ''
	classVariableNames: 'WellKnownObjectsDictionary'
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLNilTrueFalseClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for nil, true and false. They have a special treatment because they are so common in the image.!

!FLNilTrueFalseClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/19/2010 12:20'!
id

	^4! !

!FLNilTrueFalseClusterSerializer classSide methodsFor: 'class initialization' stamp: 'MartinDias 10/19/2010 12:22'!
initialize 

	super initialize.
	WellKnownObjectsDictionary := Dictionary new
		add: 1 -> nil;
		add: 2 -> true;
		add: 3 -> false;
		yourself.! !

!FLNilTrueFalseClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 12:21'!
materializeFrom: aReadStream
	
	^ WellKnownObjectsDictionary at: aReadStream next
	! !

!FLNilTrueFalseClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/19/2010 13:30'!
serialize: aWellKnownObject on: aWriteStream

	aWriteStream nextPut: (WellKnownObjectsDictionary keyAtValue: aWellKnownObject)! !

!FLPrimitiveClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:28'!
materializeAllWith: aMaterializer 
	"Template method that let to the subclass the materialization using the received materializer"
	
	| instanceCount |
	instanceCount := aMaterializer nextInteger.

	instanceCount timesRepeat: 
		[ aMaterializer addMaterialized: 
			(self materializeFrom: aMaterializer stream) ].! !

!FLPrimitiveClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:30'!
materializeFrom: aReadStream
	"Hook method that each subclass should customize for its special way of materializing"
	
	self subclassResponsibility ! !

!FLPrimitiveClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:29'!
serialize: anObject on: aWriteStream
	"Hook method that each subclass should customize for its special way of serializing"
	
	self subclassResponsibility! !

!FLPrimitiveClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:28'!
serializeAll: aCollection with: aSerializer
	"Template method that let to the subclass the serialization of each instance received"
	
	aSerializer nextPutInteger: aCollection size.
	aCollection do: [ :instance | self serialize: instance on: aSerializer stream ]! !

FLPrimitiveClusterSerializer subclass: #FLReferencedClassClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLReferencedClassClusterSerializer commentStamp: 'MartinDias 11/15/2010 11:20' prior: 0!
I am a serializer for classes that should be present when materializing. In other words, it only serializes a reference to the class, using its name.!

!FLReferencedClassClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 11/11/2010 21:01'!
id
	^ 81! !

!FLReferencedClassClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 3/19/2011 12:42'!
materializeFrom: aReadStream
	| className |
	className := aReadStream nextString asSymbol .
	^ Smalltalk globals 
		at: className
		ifAbsent: [ ClassNotFound new signal:  'Class', className, ' not found at materialization' ]! !

!FLReferencedClassClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/11/2010 21:00'!
serialize: aClass on: aWriteStream
	
	aWriteStream nextStringPut: aClass name asString! !

FLPrimitiveClusterSerializer subclass: #FLRemoteStringClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLRemoteStringClusterSerializer commentStamp: '<historical>' prior: 0!
My subclasses serialize instances of RemoteString. I centralize two possible configurations in this way:

- Full (default): the string is entirely written on the stream and then restored on the changes file.
- Light: Just the sourcePointer is serialized. Only useful when the RemoteString existed previously in the image.

Use:

The class messages #setFullSerialization and #setLightSerialization ses the current configuration, which is retrieved with #configuredInstance.

!

FLRemoteStringClusterSerializer subclass: #FLFullRemoteStringClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLFullRemoteStringClusterSerializer commentStamp: '<historical>' prior: 0!
I serialize and materialize RemoteStrings in the Full way: the string is entirely written on the stream and then restored on the changes file.
!

!FLFullRemoteStringClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/21/2011 17:21'!
id

	^63! !

!FLFullRemoteStringClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/21/2011 17:05'!
materializeFrom: aReadStream

	self flag: #todo. "Awful: File number 2 is hardcoded in the whole system."
	^ RemoteString newString: aReadStream nextString onFileNumber: 2.! !

!FLFullRemoteStringClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/21/2011 17:05'!
serialize: aRemoteString on: aWriteStream

	aWriteStream nextStringPut: aRemoteString string ! !

FLRemoteStringClusterSerializer subclass: #FLLightRemoteStringClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLLightRemoteStringClusterSerializer commentStamp: '<historical>' prior: 0!
I serialize and materialize RemoteStrings in the Light way: Just the sourcePointer is stored. Only useful when materializing a RemoteString that had existed previously in the image.
!

!FLLightRemoteStringClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/21/2011 17:21'!
id

	^64! !

!FLLightRemoteStringClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/21/2011 17:12'!
materializeFrom: aReadStream

	^ RemoteString new setSourcePointer: ((aReadStream nextWord << 16) bitOr: aReadStream nextWord).! !

!FLLightRemoteStringClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 3/21/2011 17:10'!
serialize: aRemoteString on: aWriteStream

	| source |
	source := aRemoteString sourcePointer.
	aWriteStream
		nextWordPut: (source >> 16);
		nextWordPut: source.
! !

!FLRemoteStringClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 3/21/2011 17:38'!
configuredInstance
	"Returns the instance representing the current configuration: A remoteString can be serialized full of light. Full is the default."

	^ instance ifNil: [ instance := FLFullRemoteStringClusterSerializer instance ]
! !

!FLRemoteStringClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/21/2011 17:39'!
setFullSerialization

	instance := FLFullRemoteStringClusterSerializer instance! !

!FLRemoteStringClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 3/21/2011 17:39'!
setLightSerialization

	instance := FLLightRemoteStringClusterSerializer instance! !

FLPrimitiveClusterSerializer subclass: #FLSmallIntegerClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLSmallIntegerClusterSerializer commentStamp: '<historical>' prior: 0!
I don't have so much sense as a class, but I know which of my subclasses corresponds to each SmallInteger.!

FLSmallIntegerClusterSerializer subclass: #FLNegativeSmallIntegerClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLNegativeSmallIntegerClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for negative SmallInteger instances.!

!FLNegativeSmallIntegerClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:26'!
id
	^10! !

!FLNegativeSmallIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 21:39'!
materializeFrom: aReadStream

	| hi mid1 mid2 lo |
	hi := aReadStream next.
	mid2 := aReadStream next.
	mid1 := aReadStream next.
	lo := aReadStream next.
	^ ((hi bitShift: 24) + (mid2 bitShift: 16) + (mid1 bitShift: 8) + lo) negated! !

!FLNegativeSmallIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/12/2010 10:38'!
serialize: anInteger on: aWriteStream

	| aPositiveInteger |
	aPositiveInteger := anInteger abs.
	aWriteStream nextPut: (aPositiveInteger bitShift: -24).
	aWriteStream nextPut: ((aPositiveInteger bitShift: -16) bitAnd: 255).
	aWriteStream nextPut: ((aPositiveInteger bitShift: -8) bitAnd: 255).
	aWriteStream nextPut: (aPositiveInteger bitAnd: 255).! !

FLSmallIntegerClusterSerializer subclass: #FLPositiveSmallIntegerClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLPositiveSmallIntegerClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for positive SmallInteger instances.!

!FLPositiveSmallIntegerClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:26'!
id
	^9! !

!FLPositiveSmallIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/15/2010 10:51'!
materializeFrom: aReadStream

	| hi mid1 mid2 lo |
	hi := aReadStream next.
	mid2 := aReadStream next.
	mid1 := aReadStream next.
	lo := aReadStream next.
	^ (((hi bitShift: 24) bitXor: (mid2 bitShift: 16)) bitXor: (mid1 bitShift: 8)) bitXor: lo! !

!FLPositiveSmallIntegerClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/24/2010 00:48'!
serialize: anInteger on: aWriteStream

	aWriteStream nextPut: (anInteger bitShift: -24).
	aWriteStream nextPut: ((anInteger bitShift: -16) bitAnd: 255).
	aWriteStream nextPut: ((anInteger bitShift: -8) bitAnd: 255).
	aWriteStream nextPut: (anInteger bitAnd: 255).! !

!FLSmallIntegerClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/28/2010 14:27'!
for: anSmallInteger
	"Returns the right subclass instance"

	((anSmallInteger bitAnd: 16rFFFF) = anSmallInteger)
		ifTrue: [ ^ FLUInt16ClusterSerializer instance ].

	anSmallInteger > 0 
		ifTrue: [ ^ FLPositiveSmallIntegerClusterSerializer instance ]
		ifFalse: [ ^ FLNegativeSmallIntegerClusterSerializer instance ]! !

FLSmallIntegerClusterSerializer subclass: #FLUInt16ClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLUInt16ClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for SmallInteger instances between 0 and 16rFFFF.!

!FLUInt16ClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 12/22/2010 12:21'!
id
	^90! !

!FLUInt16ClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/22/2010 12:20'!
materializeFrom: aReadStream

	| hi lo |
	hi := aReadStream next.
	lo := aReadStream next.
	^ (hi bitShift: 8) bitXor: lo! !

!FLUInt16ClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/22/2010 13:16'!
serialize: anInteger on: aWriteStream

	aWriteStream nextPut: (anInteger bitShift: -8).
	aWriteStream nextPut: (anInteger bitAnd: 255).! !

FLPrimitiveClusterSerializer subclass: #FLSymbolClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLSymbolClusterSerializer commentStamp: '<historical>' prior: 0!
I am a serializer for Symbol instances.!

!FLSymbolClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 10/12/2010 13:26'!
id

	^3! !

!FLSymbolClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/26/2010 14:22'!
materializeFrom: aReadStream

	^ aReadStream nextString asSymbol! !

!FLSymbolClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/26/2010 14:22'!
serialize: aSymbol on: aWriteStream

	aWriteStream nextStringPut: aSymbol asString! !

FLClusterSerializer subclass: #FLRawArrayedCollectionClusterSerializer
	uses: FLTClassParameter
	instanceVariableNames: 'parameterClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLRawArrayedCollectionClusterSerializer commentStamp: 'MartinDias 4/19/2011 14:02' prior: 0!
I am a cluster serializer for ArrayedCollections subclasses that are raw bits and word-like. 

I use the ArrayedCollection messages #writeOn: and #newFromStream: for serialize and materialize instances.!

FLRawArrayedCollectionClusterSerializer class
	uses: FLTClassParameter classTrait
	instanceVariableNames: ''!

!FLRawArrayedCollectionClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/19/2011 12:47'!
id

	^ 60! !

!FLRawArrayedCollectionClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 4/19/2011 12:45'!
materializeAllWith: aMaterializer

	| arrayedCollectionClass numberOfCollections |
	arrayedCollectionClass := aMaterializer nextReferencedObject.
	numberOfCollections := aMaterializer nextInteger.
	numberOfCollections timesRepeat: [ 
		aMaterializer addMaterialized: (arrayedCollectionClass newFromStream: aMaterializer stream) ]! !

!FLRawArrayedCollectionClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 4/19/2011 12:49'!
parameterClass

	^ parameterClass! !

!FLRawArrayedCollectionClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 4/19/2011 12:49'!
parameterClass: aClass

	parameterClass := aClass ! !

!FLRawArrayedCollectionClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 4/19/2011 12:52'!
referencesOf: anArrayedCollection do: aBlock
	"Evaluates the received block with each object referenced by anObject"
	
	aBlock value: self parameterClass  ! !

!FLRawArrayedCollectionClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 4/19/2011 12:44'!
serializeAll: arrayedCollections with: aSerializer

	aSerializer nextPutReferenceTo: self parameterClass.
	aSerializer nextPutInteger: arrayedCollections size.
	arrayedCollections do: [ :anArrayedCollection |
		anArrayedCollection writeOn: aSerializer stream ]! !

Object subclass: #FLInstanceVariablesMapping
	instanceVariableNames: 'mapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLInstanceVariablesMapping commentStamp: 'MartinDias 11/15/2010 11:40' prior: 0!
I am used to materialize instance variables in an object, tolerating "class shape changing".

Cases tolerated are:
- instance variable added
- instance variable order change
- instance variable removed

Note: This class may be refactorized, because it looks a bit weird.!

!FLInstanceVariablesMapping classSide methodsFor: 'instance creation' stamp: 'MartinDias 11/12/2010 00:58'!
currentClass: aClass originalInstanceVariableNames: originalInstanceVariableNames 
	
	^ self basicNew
		initializeWith: originalInstanceVariableNames and: aClass;
		yourself.! !

!FLInstanceVariablesMapping methodsFor: 'initialize-release' stamp: 'MartinDias 11/12/2010 00:58'!
initializeWith: originalInstanceVariableNames and: currentClass

	super initialize.
	
	mapping := originalInstanceVariableNames 
		collect: [ :name | currentClass instVarIndexFor: name ifAbsent: [ nil ] ].

! !

!FLInstanceVariablesMapping methodsFor: 'materialize' stamp: 'MartinDias 11/12/2010 01:17'!
materializeReferencesOf: anObject with: aMaterializer 

	1 to: mapping size 
		do: [ :index | 
			| referencedObject mappedIndex |
			
			referencedObject := aMaterializer nextReferencedObject.
			mappedIndex := mapping at: index.
			
			mappedIndex ifNotNil: [ 
					anObject 
						instVarAt: mappedIndex 
						put: referencedObject ] ].! !

Object subclass: #FLMaterializer
	instanceVariableNames: 'stream referenceMaterializingActions materializedInstances integerMaterializer materializedRoot clusterCount postReferenceMaterializingActions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterializer commentStamp: '<historical>' prior: 0!
I am a binary object materializer. 

By sending me #materialize, I will recreate previously serialized objects by FLSerializer from a byte stream. The results are obtained sending #materializedRoots.

!

!FLMaterializer class methodsFor: 'creation' stamp: 'MartinDias 9/20/2010 20:34'!
on: aReadStream

	^self new
		stream: aReadStream; 
		yourself
! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/13/2010 11:31'!
addAllMaterialized: aCollection
	
	materializedInstances addAll: aCollection ! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/13/2010 11:24'!
addMaterialized: anObject
	
	materializedInstances add: anObject ! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 10/12/2010 16:12'!
clusterSerializerFor: id

	^ FLClusterSerializer forId: id.! !

!FLMaterializer methodsFor: 'initialize-release' stamp: 'MartinDias 3/20/2011 22:20'!
initialize 
	
	super initialize.
	referenceMaterializingActions := OrderedCollection new.
	postReferenceMaterializingActions := OrderedCollection new.! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MartinDias 11/25/2010 19:14'!
materialize

	self materializeHeader.
	self materializeInstances.
	self materializeReferences.
	self materializeRoot.! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 11/25/2010 12:39'!
materializeCluster

	(self clusterSerializerFor: self nextSerializerId)
		materializeAllWith: self! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 2/26/2011 04:22'!
materializeHeader

	integerMaterializer := self clusterSerializerFor: self nextSerializerId.
	materializedInstances := OrderedCollection new: self nextInteger.
	clusterCount := self nextInteger.
! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 11/25/2010 19:16'!
materializeInstances

	clusterCount timesRepeat: [self materializeCluster].
! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 3/20/2011 22:20'!
materializeReferences

	referenceMaterializingActions
		do: [ :aBlock | aBlock value ].
	postReferenceMaterializingActions
		do: [ :aBlock | aBlock value ]
! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 10/28/2010 10:39'!
materializeRoot

	materializedRoot := self nextReferencedObject! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 10/12/2010 21:39'!
nextInteger
	^ integerMaterializer materializeFrom: stream! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 10/12/2010 21:39'!
nextReferencedObject 

	| referenceIndex |
	referenceIndex := integerMaterializer materializeFrom: stream.
	^ materializedInstances at: referenceIndex.! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 11/25/2010 12:38'!
nextSerializerId

	^ stream next! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 3/20/2011 22:19'!
postReferenceMaterializingActionsAdd: aBlock
	"Adds an action for being valued at the references materializing step"
	
	postReferenceMaterializingActions add: aBlock ! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/15/2010 10:47'!
referenceMaterializingActionsAdd: aBlock
	"Adds an action for being valued at the references materializing step"
	
	referenceMaterializingActions add: aBlock ! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/28/2010 10:51'!
root

	^ materializedRoot ! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2010 19:50'!
stream
	^ stream! !

!FLMaterializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 15:23'!
stream: aStream
	stream:= aStream.! !

Object subclass: #FLObjectAnalyzer
	instanceVariableNames: 'analyzer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Analyzer'!

!FLObjectAnalyzer commentStamp: '<historical>' prior: 0!
The purpose of my hierarchy is to collaborate with an Analyzer to traverse a graph of objects assigning to each of them an adequate FLClusterSerializer.!

FLObjectAnalyzer subclass: #FLClassPoolAssociationAnalyzer
	instanceVariableNames: 'notExternalObjects'
	classVariableNames: 'ClassPoolBindings'
	poolDictionaries: ''
	category: 'Fuel-Core-Analyzer'!

!FLClassPoolAssociationAnalyzer commentStamp: '<historical>' prior: 0!
I analyze variable class variable associations, i.e. bindings that belong to the class pool of a Class existent on Smalltalk globals. !

!FLClassPoolAssociationAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/27/2011 06:55'!
classAdded: anEvent

	anEvent item classPool associationsDo: [ :aBinding | 
			ClassPoolBindings at: aBinding put: anEvent item ]


! !

!FLClassPoolAssociationAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/27/2011 06:55'!
classModified: anEvent
	"Refresh bindings dictionary when the class variables are modified."

	anEvent item isBehavior ifFalse: [ ^self ].
	anEvent areClassVarsModified ifFalse: [ ^self ].
	anEvent oldItem classPool associationsDo: [ :aBinding | 
			ClassPoolBindings removeKey: aBinding ifAbsent: [] ].
	anEvent item classPool associationsDo: [ :aBinding | 
			ClassPoolBindings at: aBinding put: anEvent item ].
! !

!FLClassPoolAssociationAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/27/2011 06:54'!
classRemoved: anEvent

	anEvent item classPool associationsDo: [ :association | 
		ClassPoolBindings
			removeKey: association
			ifAbsent: []]! !

!FLClassPoolAssociationAnalyzer classSide methodsFor: 'class initialization' stamp: 'MarianoMartinezPeck 12/2/2010 15:21'!
initialize

	super initialize.
	self initializeClassPoolBindings.
	self initializeListeners.
! !

!FLClassPoolAssociationAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 2/26/2011 03:07'!
initializeClassPoolBindings
	ClassPoolBindings := Dictionary new.
	Smalltalk allClassesAndTraitsDo: [ :aClass | 
		aClass classPool associationsDo: [ :aBinding | 
			ClassPoolBindings at: aBinding put: aClass ] ].
	ClassPoolBindings! !

!FLClassPoolAssociationAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 3/26/2011 16:39'!
initializeListeners

	self flag: #todo. "also unregistrer? another better way of doing this?"

	SystemChangeNotifier uniqueInstance 
		notify: self
			ofSystemChangesOfItem: #class
			change: #Removed
			using: #classRemoved:;
		notify: self
			ofSystemChangesOfItem: #class
			change: #Added
			using: #classAdded:;
		notify: self
			ofSystemChangesOfItem: #class
			change: #Modified
			using: #classModified:! !

!FLClassPoolAssociationAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/27/2011 13:21'!
analyzes: anObject
	^ anObject isVariableBinding and: [ (self isClassVariableBinding: anObject) and: [ self isExternal: anObject ] ]! !

!FLClassPoolAssociationAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 3/26/2011 17:15'!
initialize

	super initialize.
	notExternalObjects := OrderedCollection new.! !

!FLClassPoolAssociationAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/27/2011 13:21'!
isClassVariableBinding: anObject
	^ ClassPoolBindings includesKey: anObject! !

!FLClassPoolAssociationAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/27/2011 13:20'!
isExternal: anObject
	^ (notExternalObjects includes: (ClassPoolBindings at: anObject)) not! !

!FLClassPoolAssociationAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 17:14'!
notExternalObjects
	^ notExternalObjects! !

!FLClassPoolAssociationAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 17:14'!
notExternalObjects: anObject
	notExternalObjects := anObject! !

!FLClassPoolAssociationAnalyzer methodsFor: 'evaluating' stamp: 'MarianoMartinezPeck 12/2/2010 15:43'!
value: anObject

	| ownerClass |
	ownerClass  := ClassPoolBindings at: anObject.
	
	analyzer 
		clusters: analyzer clusteredBaselevelObjects
		add: anObject  
		serializer: (FLClassPoolAssociationClusterSerializer newFor: ownerClass)! !

FLObjectAnalyzer subclass: #FLDefaultAnalyzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Analyzer'!

!FLDefaultAnalyzer commentStamp: '<historical>' prior: 0!
I analyze objects using the serializer specified by each's #fuelSerializer message. 

I use the visitor pattern because I'm quite coupled to each serializer implementation. (TODO) Maybe this design must be changed.

Any object could be analyzed by me.!

!FLDefaultAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 10/26/2010 10:24'!
analyzes: anObject

	^ true! !

!FLDefaultAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 3/17/2011 16:43'!
classAndTraitCollection
	"Returns the proper collection for collecting classes and traits"

	self flag: #todo. "slow"
	^ SortedCollection sortBlock: [ :a :b | a printCompactHierarchy <= b printCompactHierarchy ]! !

!FLDefaultAnalyzer methodsFor: 'evaluating' stamp: 'MartinDias 10/20/2010 15:40'!
value: anObject

	anObject fuelAccept: self! !

!FLDefaultAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 3/17/2011 16:38'!
visitClassOrTrait: aClassOrTrait
	"Visits a Class or a Trait that should be fully serialized."

	analyzer
		clusters: analyzer clusteredInternalMetalevelObjects
		add: aClassOrTrait
		serializer: aClassOrTrait fuelSerializer
		collectionFactoryBlock: [ self classAndTraitCollection ]! !

!FLDefaultAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 11/12/2010 16:10'!
visitMetaclass: aMetaclass

	analyzer  
		clusters: analyzer clusteredBaselevelObjects 
		add: aMetaclass! !

!FLDefaultAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 11/11/2010 16:12'!
visitNilTrueFalse: anObject
	"Visits these singleton objects. They has a special treatment because they are so common. And also nil is special because it cannot be added to a Set like the other objects."

	analyzer 
		clusters: analyzer clusteredBaselevelObjects 
		add: anObject 
		serializer: anObject fuelSerializer 
		collectionFactoryBlock: [ OrderedCollection new ]! !

!FLDefaultAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 11/11/2010 16:11'!
visitObject: anObject
	"Visits an object in the most generic way"

	analyzer
		clusters: analyzer clusteredBaselevelObjects
		add: anObject! !

!FLDefaultAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 11/11/2010 14:22'!
visitPrimitive: anObject
	"Visits the objects who do not depend on others"

	analyzer 
		clusters: analyzer clusteredBaselevelObjects 
		add: anObject! !

FLObjectAnalyzer subclass: #FLGlobalAnalyzer
	instanceVariableNames: 'notExternalObjects'
	classVariableNames: 'GlobalObjects'
	poolDictionaries: ''
	category: 'Fuel-Core-Analyzer'!

!FLGlobalAnalyzer commentStamp: '<historical>' prior: 0!
I analyze classes and well known objects that belong to Smalltalk global dictionary.

For example, Transcript is analyzed by me.!

!FLGlobalAnalyzer classSide methodsFor: 'class initialization' stamp: 'MarianoMartinezPeck 12/2/2010 15:44'!
classAdded: anEvent

	GlobalObjects add: anEvent item

! !

!FLGlobalAnalyzer classSide methodsFor: 'class initialization' stamp: 'MarianoMartinezPeck 12/2/2010 15:31'!
classRemoved: anEvent

	GlobalObjects 
		remove: anEvent item
		ifAbsent: [ ]! !

!FLGlobalAnalyzer classSide methodsFor: 'accessing' stamp: 'MartinDias 11/25/2010 16:16'!
globalNames

	^ #(#Display #World #Smalltalk #Transcript #SourceFiles #Processor #ImageImports #TextConstants #Undeclared #SystemOrganization #Sensor #ActiveWorld)! !

!FLGlobalAnalyzer classSide methodsFor: 'accessing' stamp: 'MartinDias 11/25/2010 16:00'!
globalObjects

	^ GlobalObjects! !

!FLGlobalAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 11/25/2010 16:17'!
initialize

	super initialize.
	self initializeGlobalObjects.
	self initializeListeners.
! !

!FLGlobalAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 11/25/2010 16:16'!
initializeGlobalObjects

	GlobalObjects := (Smalltalk allClassesAndTraits asIdentitySet)
		addAll: (self globalNames collect: [ :key | Smalltalk globals at: key ]);
		yourself! !

!FLGlobalAnalyzer classSide methodsFor: 'class initialization' stamp: 'MartinDias 11/25/2010 16:30'!
initializeListeners

	self flag: #todo. "unregistrer? anocher better way of doing this?"

	SystemChangeNotifier uniqueInstance 
		notify: self
			ofSystemChangesOfItem: #class
			change: #Removed
			using: #classRemoved:;
		notify: self
			ofSystemChangesOfItem: #class
			change: #Added
			using: #classAdded:! !

!FLGlobalAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 20:39'!
analyzes: anObject

	^ (self class globalObjects includes: anObject) and: [(notExternalObjects includes: anObject) not]! !

!FLGlobalAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 3/2/2011 20:39'!
initialize

	super initialize.
	notExternalObjects := OrderedCollection new.! !

!FLGlobalAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 20:39'!
notExternalObjects
	^ notExternalObjects! !

!FLGlobalAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 3/2/2011 20:39'!
notExternalObjects: anObject
	notExternalObjects := anObject! !

!FLGlobalAnalyzer methodsFor: 'evaluating' stamp: 'MartinDias 10/20/2010 15:40'!
value: anObject

	anObject fuelAccept: self! !

!FLGlobalAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 3/2/2011 23:56'!
visitClassOrTrait: aClassOrTrait

	analyzer  
		clusters: analyzer clusteredExternalMetalevelObjects 
		add: aClassOrTrait 
		serializer: FLReferencedClassClusterSerializer instance! !

!FLGlobalAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 3/2/2011 23:56'!
visitObject: anObject

	analyzer 
		clusters: analyzer clusteredExternalMetalevelObjects
		add: anObject  
		serializer: FLGlobalClusterSerializer instance! !

FLObjectAnalyzer subclass: #FLGlobalAssociationAnalyzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Analyzer'!

!FLGlobalAssociationAnalyzer commentStamp: '<historical>' prior: 0!
I analyze variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
is analyzed by me.!

!FLGlobalAssociationAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 10/26/2010 10:24'!
analyzes: anObject

	^ anObject isVariableBinding and: [Smalltalk globals includesAssociation: anObject ].
! !

!FLGlobalAssociationAnalyzer methodsFor: 'evaluating' stamp: 'MartinDias 3/18/2011 11:57'!
value: anObject

	analyzer 
		clusters: analyzer clusteredBaselevelObjects
		add: anObject  
		serializer: FLGlobalAssociationClusterSerializer instance! !

!FLObjectAnalyzer classSide methodsFor: 'instance creation' stamp: 'MartinDias 10/20/2010 15:46'!
newWith: anAnalyzer

	^ self basicNew
		initialize;
		analyzer: anAnalyzer;
		yourself! !

!FLObjectAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 10/20/2010 15:42'!
analyzer
	^ analyzer! !

!FLObjectAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 10/20/2010 15:42'!
analyzer: anObject
	analyzer := anObject! !

!FLObjectAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 10/26/2010 10:23'!
analyzes: anObject
	"Tells if the received object is handled by this analyzer"

	self subclassResponsibility ! !

!FLObjectAnalyzer methodsFor: 'evaluating' stamp: 'MartinDias 10/25/2010 15:20'!
value: anObject
	"Executes the analysis on the object received."

	self subclassResponsibility ! !

Object subclass: #FLPair
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLPair methodsFor: 'comparing' stamp: 'MartinDias 10/15/2010 10:54'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ self right = anObject right and: [ self left = anObject left ]! !

!FLPair methodsFor: 'comparing' stamp: 'MartinDias 10/15/2010 10:54'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ self right hash bitXor: self left hash! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
left
	^ left! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
left: anObject
	left := anObject! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
right
	^ right! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
right: anObject
	right := anObject! !

Object subclass: #FLSerializer
	instanceVariableNames: 'stream instanceIndexes referenceSerializingActions integerSerializer postReferenceSerializingActions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerializer commentStamp: '<historical>' prior: 0!
I am a binary object serializer. 

By sending me #serializeAll: with a collection of objects as argument, I will write them on a byte stream, such that later a FLMaterializer instance will be able to read the stream and recreate the objects in the same or another image.!

!FLSerializer class methodsFor: 'instance creation' stamp: 'sd 9/8/2010 14:59'!
on: aWriteStream

	| serializer |
	serializer := self new.
	serializer stream: aWriteStream.
	^serializer.! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 3/20/2011 22:20'!
initialize 

	instanceIndexes := IdentityDictionary new.
	referenceSerializingActions := OrderedCollection new.
	postReferenceSerializingActions := OrderedCollection new.! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 9/27/2010 14:53'!
nextPutInteger: anInteger

	integerSerializer serialize: anInteger on: stream! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 9/27/2010 14:52'!
nextPutReferenceTo: anObject

	integerSerializer serialize: (instanceIndexes at: anObject) on: stream! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 11/25/2010 12:34'!
nextPutSerializerId: id

	stream nextPut: id! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 3/20/2011 22:19'!
postReferenceSerializingActionsAdd: aBlock
	"Adds an action for being valued at the references serializing step"
	
	postReferenceSerializingActions add: aBlock ! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 10/15/2010 10:47'!
referenceSerializingActionsAdd: aBlock
	"Adds an action for being valued at the references serializing step"
	
	referenceSerializingActions add: aBlock ! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 11/3/2010 15:28'!
serialize: anObject 
	"Serializes using the default analyzer"

	self serialize: anObject analyzer: FLAnalyzer newDefault! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 11/3/2010 15:29'!
serialize: anObject analyzer: anAnalyzer
	"Main message for serializing"

	| objectCount clusteredObjects |
	anAnalyzer analyze: anObject.

	clusteredObjects := anAnalyzer clusteredObjects.
	objectCount := anAnalyzer objectCount.
	integerSerializer := objectCount fuelSerializer.

	self serializeHeaderWith: clusteredObjects and: objectCount.
	self serializeInstances: clusteredObjects.
	self serializeReferences.
	self serializeRoot: anObject.
! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 2/26/2011 04:23'!
serializeHeaderWith: clusteredObjects and: objectCount

	self nextPutSerializerId: integerSerializer class id.
	self nextPutInteger: objectCount.
	self nextPutInteger: clusteredObjects size! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 3/21/2011 01:08'!
serializeInstances: clusteredInstances

	clusteredInstances
		do: [ :anAssociation | 
			| serializer instances |
			serializer := anAssociation key.
			instances := anAssociation value.

			instances do: [ :instance | instanceIndexes at: instance put: instanceIndexes size + 1 ].
			self nextPutSerializerId: serializer class id.
			serializer serializeAll: instances with: self.
		]! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 3/20/2011 22:19'!
serializeReferences

	referenceSerializingActions 
		do: [ :aBlock | aBlock value ].
	postReferenceSerializingActions 
		do: [ :aBlock | aBlock value ]! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 10/28/2010 10:36'!
serializeRoot: anObject

	self nextPutReferenceTo: anObject! !

!FLSerializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 14:59'!
stream
	^ stream! !

!FLSerializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 14:59'!
stream: anObject
	stream := anObject! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 9/27/2010 16:49'!
fuelAccept: aVisitor

	aVisitor visitObject: self.
! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 9/24/2010 01:36'!
fuelSerializer

	^ FLObjectClusterSerializer for: self class! !
FLClassClusterSerializer initialize!
FLClassPoolAssociationAnalyzer initialize!
FLClusterSerializer initialize!
FLGlobalAnalyzer initialize!
FLNilTrueFalseClusterSerializer initialize!
